
function [phi H] = dyadiciteration2scaleeq(h,J,maxiter,phi0);

% dyadiciteration2scaleeq - finds father function from refinement filter, using
%                           dyadic iteration
% Usage
%   phi = dyadiciteration2scaleeq(h,J,maxiter,phi0);
%   [phi H] = dyadiciteration2scaleeq(h,J,maxiter,phi0);
% Input
%   h    real vector; refinement filter (default h = [1 1])
%   J    integer scalar; number of resolution levels (default J=1)
%   maxiter integer scalar; maximum number of iterations
%   phi0 initial vector (default the identicator on [0 1[)
% Output
%   phi  father function, evaluated in the dyadic points 2^(-J)*(0:2^J*(s-1)) 
%        with s = length(h)
%   H    square sxs matrix, so that H*phik = phik

if nargin<4, phi0 = NaN; end
if nargin<3, maxiter = 100; end
if nargin<2, J = 0; end
if nargin<1, h = [1 1]; end
tol = 1.e-8;
h = 2*h/sum(h);

s = length(h);
x = (0:2^J*(s-1))/2^J;
n = length(x);
if isnan(phi0), phi0 = zeros(size(x)); phi0(x<1) = 1; end

% HI = zeros(s,s);
% for r=1:s,
%    cl = max(1,2*r-s);
%    cr = min(s,2*r-1);
%    hl = min(s,2*r-1);
%    hr = hl-cr+cl;
%    HI(r,cl:cr) = h(hl:-1:hr);
% end

H = zeros(n,n);
% H(1:2^J:n,1:2^J:n) = HI;

% phi(x) = sum_k h_k phi(2x-k)
% with h_k = h(k+1)
% For: x(i) = (i-1)/2^J
% this is: phi[(i-1)/2^J] = sum h(k+1) phi[{2(i-1)-2^J*k+1 -1}/2^J]
% So: H(i,j) = h(k+1), with j = 2(i-1)-2^J*k+1
for k=0:s-1,
   i = (1:n);
   j = 2*(i-1)-2^J*k+1;
   o = find(j>0 & j<n+1);
   i = i(o); j = j(o);
   ij = i+n*(j-1);
   H(ij) = h(k+1);
end

phi0 = column(phi0);
iter = 0;
while iter < maxiter,
   iter = iter+1;
   phi = H*phi0;
   if max(abs(phi-phi0))<tol, maxiter=iter; else, phi0 = phi; end
end
iter
