% Constructs contourlines in the constraint LS optimisation problem
% norm(Y-X*beta) s.t. norm(beta) = C with beta a bivariate vector
%
% in case where the model is overspecified, the contour lines of norm(Y-X*beta)
% are ellipses. (This includes the case of critical specification)

if exist('dosingular') ~= 1,
   dosingular = false;
end
if exist('doorthogonal') ~= 1,
   doorthogonal = false;
end
if exist('makeeps') ~= 1,
   makeeps = false;
end
if exist('makeslides') ~= 1,
   makeslides = false;
end
% if makeeps == false, makeslides = false; end
if makeslides == true, makeeps = true; end
randn('state',0);
linecolor = 'k';
fullline = {'linewidth',2,'color',linecolor};
% dottedline = {'.','linewidth',4,'color',linecolor};
dottedline = {'o','markersize',6,'markerfacecolor',linecolor,'color',linecolor};
bullet = {'o','markersize',12,'linewidth',2,'color',linecolor};
fillcolor = [1 1 1]*0.7;
beta = [2; 1];
X = [3 1;1 4;3 5;1 5];
Xsing = repmat([3 2],4,1);
% Xort = [1 0;1 0;0 1;0 1];
Xort = [1 -1;1 -1;1 1;1 1];
if dosingular == true,
   alfa = 0;
   X = alfa*Xort+(1-alfa)*Xsing;
end
if doorthogonal == true,
   X = Xort;
end
mu = X*beta;

stdev = norm(mu,1)/8;
n = size(X,1); m = size(X,2);
err = stdev*randn(n,1);
Y = mu+err;
singularproblem = (cond(X'*X) > 1.e+6);
if singularproblem == false,
   betaLS0 = (X'*X)\(X'*Y);
else
   betaLS0 = pinv(X)*Y;
   % pinv: Moore-Penrose pseudo-inverse:
   %       minimum norm solution of normal equations
   %       betaLS0 lies on the smallest ball that has an intersection with the
   %       line of solutions of the normal equations
   % COROLARY: the pseudo-inverse is the limit for lambda->0 of an
   %           l2-regularized regression
end

[Q1 R1] = qr(X,0); % economy size QR decomposition; Q1 rectangular
[Q R] = qr(X); % Q full orthogonal matrix; R upper triangular;
               % R*beta is of size n, but only the first m entries are nonzero
               % (m being length(beta))
% norm(Y-X*beta) = c <=> norm(Q'*Y-R*beta) = c <=> 
% R*beta is on a sphere with radius c. As the last n-m components of R*beta are
% zero, we can write Q'*Y = [z1;z2] where z1 and z2 are vectors of lengths m
% and n-m respectively. norm(Q'*Y) = sqrt(norm(z1)^2+norm(z2)^2). 
% (Note: this is always the case when splitting a vector into 2 subvectors)
% We have norm(z1-R1*beta)^2 = c^2-norm(z2)^2
% As a result, the minimum norm (the least squares) is obtained when z1 =
% R1*beta, i.e., beta = R1\z1;
% Hence, the equation of the ellips is also norm(R1*(beta-betaLS0)) = s
z = Q'*Y; z1 = z(1:m); z2 = z(m+1:n);

nn = 1000;
if singularproblem == true,
   nn1 = floor(nn/2); nn2 = nn-nn1-1;
   t = (1:nn1)/nn1; t = 2*t-1; t = [t NaN t(1:nn2)]; t = t/5;
   betaLS = repmat(betaLS0,1,nn) + [R1(1,2); -R1(1,1)]*t;
   % alternative: find solutions with beta_1 = 0 and beta_2 = 0 respectively:
   X1 = X(1:n,1); X2 = X(1:n,2);
   beta1 = (X1'*X1)\(X1'*Y);
   beta2 = (X2'*X2)\(X2'*Y);
   t = (0:nn1-1)/(nn1-1); t = [t NaN t(1:nn2)];
   betaLS = [beta1;0]*t+[0;beta2]*(1-t);
   % --> This is the limiting case of ell_1-penalization !!

   % We impose that beta = betaLS + s*tt with norm(R1*(beta-betaLS))) = s
   % where betaLS is one of the solutions of the normal equation
   % so, norm(R1*s*tt) = s
   % or: norm(R1*tt) = 1
   % as R1(2,1:2) = [0 0], this reduces to
   % R1(1,1)*tt(1) + R1(1,2)*tt(2) = +/- 1
   % Several options -> we choose to avoid devision by either R1(1,1) or
   % R1(1,2), since we cannot prevent them from being arbitrarily close to 0
   tt = [R1(1,1); R1(1,2)]/norm(R1(1,1:2))^2*[ones(1,nn1) NaN -ones(1,nn2)];
   R1 = eye(m);
else
   % betaLS = R1\z1
   t = (1:nn)/nn*2*pi;
   tt = [cos(t); sin(t)];
   betaLS = R1\repmat(z1,1,nn);
end

figure(2)

u = (1:nn)/nn*2*pi;
l2ball = [cos(u); sin(u)];
plot(betaLS0(1),betaLS0(2),bullet{:})
hold on
if singularproblem == true,
   plot(betaLS(1,:),betaLS(2,:),fullline{:})
   C = norm(betaLS0);
   plot(C*l2ball(1,1:nn),C*l2ball(2,1:nn),fullline{:})
   fill(C*l2ball(1,1:nn),C*l2ball(2,1:nn),fillcolor*1.2)
end

% Now, find least squares under the constraint that norm(beta) < C
% If X'*X is singular, this constraint is necessary to identify a solution. If
% X'*X is regular, the new beta will be closer to the origin (because of the 
% constraint)
%
% First, for the 2-norm:
% find the intersection between the ellips norm(R1(beta-betaLS0)) = s and the
% circle norm(beta) = C
% The solution depends on s
% We find the s such that there is a single solution, i.e., discriminant is 0
%
% beta2^2 = C^2-beta1^2
% d = 
% ellips: see http://mathworld.wolfram.com/Ellipse.html
%
% Solution for given C is difficult. We compute solution for given lambda, and
% then find corresponding C

L = 35;lambdamax = 1000;
ll = polyline(((0:L)/L).^2,[0 0.2 0.999 1],[0 0.04 0.4 1])*lambdamax;
betaL2 = zeros(m,L);
XTX = X'*X;
I = eye(m);
SSEell2 = zeros(1,L);
for l = 1:L,
   lambda = ll(l);
   betaL2(1:m,l) = (X'*X+lambda*I)\(X'*Y);
   SSEell2(l) = norm(Y-X*betaL2(1:m,l))^2;
end
plot(betaL2(1,1:L),betaL2(2,1:L),dottedline{:})

s = 5;
l = min(find(SSEell2-norm(Y-X*betaLS0)^2 > s^2))
if ~isempty(l),
   plot(betaL2(1,l),betaL2(2,l),bullet{:})
   s = sqrt(SSEell2(l)-norm(Y-X*betaLS0)^2)
   betas = betaLS+R1\(s*tt);
   plot(betas(1,1:nn),betas(2,1:nn),fullline{:})

   C = norm(betaL2(1:m,l));
   plot(C*l2ball(1,1:nn),C*l2ball(2,1:nn),fullline{:})
   fill(C*l2ball(1,1:nn),C*l2ball(2,1:nn),fillcolor)

   plot(betaL2(1,1:L),betaL2(2,1:L),dottedline{:})
   plot(betaL2(1,l),betaL2(2,l),bullet{:})
end

for s = 1:4,
   betas = betaLS+R1\(s*tt);
   plot(betas(1,1:nn),betas(2,1:nn),fullline{:})
end


hold off

axis equal
axis off
axy = axis; 
hwratio = (axy(4)-axy(3))/(axy(2)-axy(1));
axy(1) = min(axy(1),axy(3)); axy(3) = axy(1); 
axy(2) = axy(1)+(axy(2)-axy(1))*1.1;
axy(4) = axy(3) + hwratio*(axy(2)-axy(1));
axis(axy)
drawaxes

if makeslides == true,
   makefigtransparent
end

if singularproblem == true,
   fname = 'l2regLSsing.eps';
else
   fname = 'l2regLS.eps';
end
if makeeps == true, print('-depsc',fname), end

% Now, we do L1-regularization (lasso)

figure(1)
plot(betaLS0(1),betaLS0(2),bullet{:})
hold on
if singularproblem == true,
   plot(betaLS(1,:),betaLS(2,:),fullline{:})
end

L = 35;lambdamax = 200;
betaL1 = zeros(m,L);
XTX = X'*X;
I = eye(m);
SSEell1 = zeros(1,L);
X1 = X(1:n,1); X2 = X(1:n,2);
XTY = X'*Y;
lambda1 = min(abs(XTY));
ll = polyline((0:L)/L,[0 0.75 1],[0 lambda1 lambdamax]);
for l = 1:L,
   lambda = ll(l);
   beta1 = (X1'*X1)\ST(X1'*Y,lambda);
   beta2 = (X2'*X2)\ST(X2'*Y,lambda);
   if singularproblem == true,
      betaL1(1:m,l) = zeros(m,1);
   else
      betaL1(1:m,l) = (XTX)\ST(XTY,lambda);
   end
   SSEell1(l) = norm(Y-X*betaL1(1:m,l))^2;
   if SSEell1(l) > norm(Y-X*[beta1; 0])^2,
      betaL1(1:m,l) = [beta1; 0];
      SSEell1(l) = norm(Y-X*betaL1(1:m,l))^2;
   end
   if SSEell1(l) > norm(Y-X*[0; beta2])^2,
      betaL1(1:m,l) = [0; beta2];
      SSEell1(l) = norm(Y-X*betaL1(1:m,l))^2;
   end
end
plot(betaL1(1,1:L),betaL1(2,1:L),dottedline{:})

s = 5;
if singularproblem == 1, s = 0; end
l = min(find(SSEell1-norm(Y-X*betaLS0)^2 > s^2-eps))
if ~isempty(l),
   plot(betaL1(1,l),betaL1(2,l),bullet{:})
   s = sqrt(SSEell1(l)-norm(Y-X*betaLS0)^2)
   betas = betaLS+R1\(s*tt);
   plot(betas(1,1:nn),betas(2,1:nn),fullline{:})

   C = norm(betaL1(1:m,l),1);
   l1ball = [1 0 -1 0 1;0 1 0 -1 0];
   vv = 5;
   plot(C*l1ball(1,1:vv),C*l1ball(2,1:vv),fullline{:})
   fill(C*l1ball(1,1:vv),C*l1ball(2,1:vv),fillcolor)

   plot(betaL1(1,1:L),betaL1(2,1:L),dottedline{:})
   plot(betaL1(1,l),betaL1(2,l),bullet{:})
end

l1 = find(betaL1(1,1:L)==0 | betaL1(2,1:L)==0);
l1l = length(l1);
if singularproblem == true,
   s = 5;
   l = min(find(SSEell1-norm(Y-X*betaLS0)^2 > s^2-eps))
end
i = min(l1l,10);
if i > 0,
   l = max(l,l1(i));
   plot(betaL1(1,l),betaL1(2,l),bullet{:})
   s = sqrt(SSEell1(l)-norm(Y-X*betaLS0)^2)
   betas = betaLS+R1\(s*tt);
   plot(betas(1,1:nn),betas(2,1:nn),fullline{:})
   C = norm(betaL1(1:m,l),1);
   l1ball = [1 0 -1 0 1;0 1 0 -1 0];
   vv = 5;
   plot(C*l1ball(1,1:vv),C*l1ball(2,1:vv),fullline{:})
   fill(C*l1ball(1,1:vv),C*l1ball(2,1:vv),fillcolor*0.8)
end

if betaL1(1,l) == 0,
   plot(betaL1(1,l1)-2.e-2,betaL1(2,l1),dottedline{:})
else
   plot(betaL1(1,l1),betaL1(2,l1)-2.e-2,dottedline{:})
end
for s = 1:4,
   betas = betaLS+R1\(s*tt);
   plot(betas(1,1:nn),betas(2,1:nn),fullline{:})
end

hold off
axis off
% We keep the same axes as for the l2-constraint problem
axis(axy)
drawaxes


if makeslides == true,
   makefigtransparent
end

if singularproblem == true,
   fname = 'l1regLSsing.eps';
else
   fname = 'l1regLS.eps';
end
if makeeps == true, print('-depsc',fname), end

