
function [Q R P] = QRbyHouseholderB(A,m1n,B);

% QRbyHouseholderB -- finds orthogonal Q and upper triangular R such that
%       Q'*A+B = R
%  Usage
%    [Q R P] = QRbyHouseholderB(A);
%    [Q R P] = QRbyHouseholderB(A,k);
%    [Q R P] = QRbyHouseholderB(A,B);
%    [Q R P] = QRbyHouseholderB(A,k,B);
%    [Q R P] = QRbyHouseholderB(A,B,k);
%  Inputs
%    A      rectangular or squared matrix; let [n m] = size(A)
%    B      matrix, size(A)=size(B), default: zeros(size(A))
%    k      number of columns in A whose elements below the diagonal to be
%           transformed into zeros; default: min(n-1,m), where [n m] = size(A)
%  Outputs
%    Q      orthogonal matrix of size n x n, with n the # rows in A
%    R      upper triangular matrix (up to column k) with size(R) = size(A)
%    P      cell array of Householder reflections, so that
%           Q = P{1}*P{2}*...*P{k}
%  Description
%  Note
%  See also

maxit = 100;
tol = 1.e-10;
[n m] = size(A);
if nargin < 2, m1n = min([n-1 m]); B = zeros(size(A)); end
if nargin == 2, 
   if max(size(m1n)) > 1, 
      B = m1n; m1n = min([n-1 m]); 
   else
      B = zeros(size(A));
   end
end
if max(size(m1n)) > 1, realm1n = B; B = m1n; m1n = realm1n; end
if any(size(B)~=size(A)), error('dimension mismatch A and B'), end
m1n = min([m1n n-1 m]);

I = eye(n); Q=I; R=A+B; P = cell(1,m1n);
for k = 1:m1n,
   RminB = R-B;
   L = RminB(1:k,1:k-1)';
   r = RminB(1:n,1:k-1)'*RminB(1:n,k)+RminB(k+1:n,1:k-1)'*B(k+1:n,k);
   % A = eye(k); b = zeros(1,k);
   c = norm(RminB(1:n,k))^2-norm(B(k+1:n,k))^2;
   [RminBknew it] = linquadsystNR(eye(k),zeros(1,k),c,L,r,RminB(1:k,k),maxit);
   RminBknew = [RminBknew; -B(k+1:n,k)];
   vk = RminB(1:n,k)-RminBknew;
   Pk = I - 2*vk*((vk'*vk)\vk');
   Q = Q*Pk';
   RminB = Pk*RminB; R=RminB+B; R(abs(R)<tol) = 0;
   P{k} = Pk;
end
