
function [Q R] = gramschmidt(A,Q)

% ThreshLab/General/gramschmidt.m -- ThreshLab's Gram Schmidt orthogonalization
%  Usage
%    Q = gramschmidt(A)
%    [Q R] = gramschmidt(A)
%    [Q R] = gramschmidt(A,Q)
%  Input
%    A   matrix
%    Q   matrix with orthogonal columns
%  Output
%    Q   matrix with same size as A
%    R   upper triangular matrix of squared size, equal to the number of
%        columns in A
%   Description
%    if size(A) = [n m], then the first min(m,n) columns of Q are orthogonal 
%    in absence of input Q, the output satisfies A = QR
%    if [Qout R] = gramschmidt(A,Qin), then Qout*R = [Qin A]

eps1 = 1.e-10;

[n m] = size(A);
if nargin < 2,
   Q = zeros(size(A));
   R = eye(m);
   alfak = norm(A(1:n,1));
   if alfak > eps,
      Q(1:n,1) = A(1:n,1)/alfak;
   else
      alfak = 0;
   end
   R(1,1) = alfak;
   k1 = 2;
else
   [nQ mQ] = size(Q);
   if nQ ~= n,
      error('dimension mismatch input matrices')
   end
   if max(max(abs(Q'*Q-eye(mQ)))) > eps1,
      warning('input matrix Q must be orthogonal')
   end
   A = [Q A];
   Q = [Q zeros(n,m)];
   m = m+mQ;
   R = eye(m);
   k1 = mQ+1;
end
for k = k1:m
   Rk = eye(m);
   c = Q(1:n,1:k-1)'*A(1:n,k);
   V = A(1:n,k) - Q(1:n,1:k-1)*c;
   alfak = norm(V);
   if alfak < eps1,
      alfak = 0;
      Q(1:n,k) = 0;
   else
      Q(1:n,k) = V/alfak;
   end
   Rk(1:k-1,k) = c; Rk(k,k) = alfak;
   R = Rk*R;
end

% Copyright (c) Maarten Jansen
% 
% This software is part of ThreshLab and is copyrighted material. 
