
function [pos, level] = showasgreyleveltree(x,varargin)

% showasgreyleveltree - ThreshLab/Plots -- displays binary tree structured data
%   Usage
%     showasgreyleveltree(x,varargin)
%     showasgreyleveltree(x,subtree,varargin)
%     showasgreyleveltree(n,subtree,varargin)
%
%  Optional inputs
%     subtree              integer vector of length < length(x) with indices 
%                          into x 
%     n                    integer so that all elements in subtree are below n
%     x                    real vector, length n, whose value define color
%                          intensity in tree (see description below)
%     parents              integer vector of length(x)
%                          default is floor((1:n)/2); which represents a full
%                          binary tree
%     children             2x length(x) integer matrix
%     'fullcolor',c        c is a string or a triple of numbers in [0,1]
%                          indicating the color (RGB value)
%     'mincolor',r         r is a number in [0,1], indicating the minimum grey
%                          value (i.e., the minimum distance between a displayed
%                          color and full white)
%     'onecolor'           use a single color only for all nodes (default value)
%     'onecolor',c         use a single color only for all nodes, given by c (a
%                          string or a triple of numbers in [0,1])
%     'onefillcolor'       use a single color only for filling the nodes (the
%                          edges of the nodes are in black)
%     'onefillcolor',c     use a single color only for filling the nodes, given
%                          by c
%     'markeredgecolor',c  as in Matlab's plot
%     'markersize',n       as in Matlab's plot
%     'linewidth'n         as in Matlab's plot
%     'noedges'            display nodes, but no edges in between them
%  Outputs
%     void or [pos level]
%     pos     positions of nodes within their levels
%     level   levels of nodes
%  Description
%     displays a binary subtree defined by x(subtree) with greyvalues defined by
%     (x-min(x))/(max(x)-min(x))
%     if x is replaced by n, then a subtree of a full binary tree with n
%     elements is displayed, using a single gray value
%     The first element of x is displayed as root node, the i-th element comes
%     at level j, position k, where (in the default settings) 
%     j = floor(log2(i)); k = i-2^j;
%  See also
%     help allbinarysubtrees
%     help randombinarysubtree
%     help maxmassubtree

%%% parse the input variables
n = length(x);
if n == 1, n = x; x = ones(n,1); end
maxlevels = Inf;
subtree = NaN;
parents = NaN;
nodesidecolor = 'k';
vertexsize = 12;
edgewidth = 3;
fullcolor = [0 0 0]; % black
minedgecolor = 0.3; % minimum grey level 
                % (i.e., no color closer to white than this value)
onecolor = false;
noedges = false;
nvarargin = length(varargin);
k = 1;
if nvarargin>0, if isnumeric(varargin{1}), 
   subtree = varargin{1}; k=2;
   if nvarargin>1, if isnumeric(varargin{2}),
      parents = varargin{2}; k = 3;
   end, end
end, end
if any(size(subtree)==n) | any(subtree==0) | hasdoubles(subtree),
   change = subtree; subtree = parents; parents = change;
end
if isnan(subtree), subtree = (1:n); end
if isnan(parents), parents = floor((1:n)/2); end
[parents children] = parentschildren(parents,'cell');
c0 = max(find(parents==0));
n = length(parents);

while k <= nvarargin, vark = varargin{k};
   if ischar(vark), switch vark,
   case {'fullcolor'},
      k = k+1;
      fullcolor = varargin{k};
   case {'mincolor','minedgecolor'},
      k = k+1;
      minedgecolor = varargin{k};
   case {'onecolor'},
      onecolor = true;
      if k+1 <= nvarargin, if iscolor(varargin{k+1}),
         k = k+1;
         fullcolor = varargin{k};
         nodesidecolor = varargin{k};
      end, end
   case {'onefillcolor','fillcolor'},
      onecolor = true;
      if k+1 <= nvarargin, if iscolor(varargin{k+1}),
         k = k+1;
         fullcolor = varargin{k};
      end, end
   case {'markeredgecolor'},
      k = k+1;
      nodesidecolor = varargin{k};
   case {'markersize','vertexsize'},
      k = k+1;
      vertexsize = varargin{k};
   case {'linewidth','edgewidth'},
      k = k+1;
      edgewidth = varargin{k};
   case {'noedges'},
      noedges = true;
   case {'maxlevels'},
      k = k+1;
      maxlevels = varargin{k};
   end, end
   k = k+1;
end

fullcolor = rgb(fullcolor);

% The value of n was initially set to length(x), later (after the
% identification of parents) changed to length(parents) (as parents defines
% the tree). Now make sure that x and subtree do not go beyond that n
x = [column(x); zeros(n,1)]; x = x(1:n);
subtree = subtree(find(subtree<=n));

% Define the color intensities in the nodes
xmax = max(x); xmin = min(x);
if xmax-xmin < eps,
   cx = column(ones(size(x)));
else
   cx = column(x-xmin)/(xmax-xmin);
end
if onecolor==true, cx = ones(size(cx)); end
white = [1 1 1];
cc = cx*fullcolor+(1-cx)*white;

%%% end parse input variables

holdstatus = ishold;

vertexoptions = {'markersize',vertexsize,'markeredgecolor',nodesidecolor};
edgeplotoptions = {'linewidth',edgewidth};

%%% Compute level (height/depth in tree) for each node
%%% + add artificial nodes whereever needed for tree representation
nnodeswithc = length(children);
children = [children; cell(n-nnodeswithc,1)];
currentlevel = find(parents==0);
level = nan(n,1); l = 0; level(currentlevel) = l; m = n;
while any(isnan(level(1:n))),
   nextlevel = [];
   for p=currentlevel, c = children{p};
      % add artificial child to nodes without children (needed to reserve a
      % position for this node and its parents between the nodes at deeper
      % levels)
      if isempty(c),
         m=m+1; c = m; children{p} = m; parents = [parents p];
         children = [children; cell(1,1)];
      end
      nextlevel = [nextlevel c];
   end
   currentlevel = nextlevel; l=l+1; level(currentlevel) = l;
end

%%% Compute horizontal position for each node
pos = zeros(1,m);
pos(currentlevel) = (1:length(currentlevel));
while l>0,
   l=l-1;
   currentlevel = find(level==l);
   for i=1:length(currentlevel),
      p=currentlevel(i); c = children{p}; pos(p) = mean(pos(c));
   end
end
   
level = level(1:n); pos = pos(1:n);
level(level>maxlevels) = NaN; % do not display deepest levels


loglevel = log(level+1);
plot(pos(subtree),loglevel(subtree),'o',vertexoptions{:})
hold on
vv = subtree(find(subtree>c0));
if noedges == false, for v = row(vv),
   pv = parents(v);
   if ismember(pv,subtree),
      e = [v pv];
      ecol = max(min(cx(e)),minedgecolor);
      ecol = ecol*fullcolor+(1-ecol)*white;
      plot(pos(e),loglevel(e),edgeplotoptions{:},'color',ecol)
   end
end, end

for v = row(subtree),
   plot(pos(v),loglevel(v),'o',vertexoptions{:},'markerfacecolor',cc(v,1:3))
end
jj = 0:max(level); ll = log(jj+1);
set(gca,'Ytick',ll,'Yticklabel',jj)

level = loglevel;
if holdstatus == false, hold off, end

if nargout == 0,
   clear pos level;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function B = iscolor(c),

B = false;
if ischar(c),
   if ismember(c,{'k','black','Black','r','red','Red','g','green','Green'}),
      B = true;
   elseif ismember(c,{'b','blue','Blue','y','yellow','Yellow'}),
      B = true;
   elseif ismember(c,{'m','magenta','Magenta','c','cyan','Cyan'}),
      B = true;
   elseif ismember(c,{'forestgreen','forestgreen','Forestgreen'}),
      B = true;
   elseif ismember(c,{'w','white','White'}),
      B = true;
   end
elseif isnumeric(c),
   if ismember(3,size(c)),
      cmin = min(min(c)); cmax = max(max(c));
      if cmin >= 0 & cmax <= 1, B = true; end
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function c = rgb(c),

if ischar(c), switch c,
case {'k','black','Black'},
   c = [0 0 0];
case {'r','red','Red'},
   c = [1 0 0];
case {'g','green','Green'},
   c = [0 1 0];
case {'b','blue','Blue'},
   c = [0 0 1];
case {'y','yellow','Yellow'},
   c = [1 1 0];
case {'m','magenta','Magenta'},
   c = [1 0 1];
case {'c','cyan','Cyan'},
   c = [0 1 1];
case {'forestgreen','forestgreen','Forestgreen'},
   c = [34 139 34]/255;
case {'w','white','White'},
   c = [1 1 1];
end, end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function B = hasdoubles(x),

x = sort(x); x = x(2:end)-x(1:end-1); B = any(x<eps); 

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
