
function V = gammanoise(alfa);

% gammanoise -- generates (pseudo) Gamma data for given vector of parameter
%  Usage
%    V = gammanoise(alfa,realrandom);
%    V = gammanoise(alfa);
%  Inputs
%    alfa         vector or matrix with positive or zero real numbers
%    realrandom   binary/logical variable; default false
%  Outputs
%    V        vector or matrix of size(alfa), containing positive real values
%  Description
%    V contains Gamma(alfa,alfa) distributed data
%        E(V) = 1 (unless alfa=0)
%      var(V) = 1./alfa
%  Notes
%    (1) gammanoise calls matlab's RAND function and therefore changes RAND's
%    state.
%    (2) Combine with poissonnoise to generate overdispersed count data:
%    V = gammanoise(alfa);
%    X = poissonnoise(V.*mu);
%    (3) W = V.*alfa./lambda will be Gamma(lambda,alfa) distributed
%    meaning that f_W(w) = lambda^alfa*w^(alfa-1)*exp(-lambda*w)/Gamma(alfa)
%  See also
%    help poissonnoise
%    help randgamma

r = floor(alfa);
ralfa = alfa - r;
Vr = erlangnoise(r);
Vralfa = zeros(size(alfa));

% Now, generate Vralfa ~ gamma(1,ralfa)
e = exp(1); p = e./(e+ralfa);
A = find(abs(ralfa) > eps); nA = length(A); % A is active set
while nA>0,
   ralfaA = ralfa(A);
   pA = p(A);
   V = rand(size(A)); U = rand(size(A)); R = rand(size(A));
   I = find(V<=pA); II = find(V>pA);
   X = zeros(size(A));
   X(I) = U(I).^(1./ralfaA(I)); X(II) = 1-log(U(II));
   Vralfa(A) = X;
   fXoverMgX = zeros(size(A));
   fXoverMgX(I) = exp(-X(I));
   fXoverMgX(II) = X(II).^(ralfaA(II)-1);
   A = find(R>fXoverMgX);
   nA = length(A);
end
non0 = find(alfa>0);
V = Vr+Vralfa; V(non0) = V(non0)./alfa(non0);



function Vr = erlangnoise(r);

% generates Vr ~ Erlang(1,r)

Vr = zeros(size(r));
rnon0 = find(r>0);
nrnon0 = length(rnon0);
while nrnon0>0,
   X = randexp(size(rnon0));
   Vr(rnon0) = Vr(rnon0)+X;
   r = r-1; rnon0 = find(r>0); nrnon0 = length(rnon0);
end
