function x = makegrid(N,gridID,varargin)

% makegrid -- ThreshLab/Tests/
%  Usage
%    x = makegrid(N,gridID,realrandom);
%    x = makegrid(N,gridID);
%    x = makegrid(N);
%    x = makegrid(N,gridID,[a b]);
%  Inputs
%    N          integer, size of output vector
%    gridID     integer or string, default 0
%    realrandom boolean, default false
%    [a b]      pair of real values: interval: a = x(1); b = x(N);
%               defaults a = 0; b = 1; except for regular grids, then default 
%               is a = 1/N
%  Outputs
%    x      real ordered vector with entries in [0,1]
%  Description
%    if gridID==0, then x = (1:N)/N;
%    if gridID==1, then x = sort(rand(1,N));
%    other values of gridID create other grids
%    if realrandom==false, then rand is reset to fixed state
%  Note
%  See also
%    help makesignal

if nargin < 2, gridID = 0; end
if ischar(gridID), switch gridID,
case {'regular','equidistant','equi','equispaced'},
   gridID = 0; a = 1/N; 
case {'uniform','randomuniform','uniformrandom'}
   gridID = 1;
case {'cantor','Cantor','devilsstaircase'},
   gridID = 666;
otherwise
   warning('gridID not recognized; makegrid uses regular grid instead')
   gridID = 0; a = 1/N;
end, end

realrandom = NaN;
a = 0; b = 1;
gridoptions = {};
nvarargin = length(varargin);
for k = 1:nvarargin, vark = varargin{k};
   if isnumeric(vark) | islogical(vark),
      dimvark = size(vark,1)*size(vark,2);
      switch dimvark,
      case 1, 
         if ismember(vark,[true false]) & isnan(realrandom),
            realrandom = vark;
         else
            gridoptions = {gridoptions{:},vark};
         end
      case 2,
         a = vark(1); b = vark(2);
      otherwise
         gridoptions = {gridoptions{:},vark};
      end
   else
      gridoptions = {gridoptions{:},vark};
   end
end
if isnan(realrandom), realrandom = false; end

if realrandom == false & gridID > 0,
   rand('seed',931316785);
end

switch gridID
case 0
   x = (0:N-1)/(N-1);
case 1
   x = sort(rand(1,N));  
case 2
   N1 = floor(N/20);
   N2 = N1 + floor(N/200);
   x(1:N1) = rand(size(1:N1));
   x(1:N1) = x(1:N1)/5;
   x(N1+1:N2) = rand(size(N1+1:N2));
   x(N1+1:N2) = 0.2 + x(N1+1:N2)*0.2;
   x(N2+1:N) = rand(size(N2+1:N));
   x(N2+1:N) = 0.4 + x(N2+1:N)*0.6;
   x = sort(x);
case 3
   N1 = floor(N/20);
   N2 = N1 + floor(N/100);
   x(1:N1) = rand(size(1:N1));
   x(1:N1) = x(1:N1)*0.2;
   x(N1+1:N2) = rand(size(N1+1:N2));
   x(N1+1:N2) = 0.2 + x(N1+1:N2)*0.2;
   x(N2+1:N) = rand(size(N2+1:N));
   x(N2+1:N) = 0.4 + x(N2+1:N)*0.6;
   x = sort(x);
case 31
% sparsely sampled interval in the middle
   N1 = floor(47*N/100);
   N2 = N1 + floor(N/20);
   N3 = N2 + floor(N/100);
   x(1:N1) = rand(size(1:N1));
   x(1:N1) = x(1:N1)*0.3;
   x(N1+1:N2) = rand(size(N1+1:N2));
   x(N1+1:N2) = 0.3 + x(N1+1:N2)*0.2;
   x(N2+1:N3) = rand(size(N2+1:N3));
   x(N2+1:N3) = 0.5 + x(N2+1:N3)*0.2;
   x(N3+1:N) = rand(size(N3+1:N));
   x(N3+1:N) = 0.7 + x(N3+1:N)*0.3;
   x = sort(x);
case 32
   % normally distributed samples
   x = sort(randn(1,N));
   x = (x-min(x))/(max(x)-min(x));
case 4
   N3 = floor(N/20);
   N2 = floor(N/100);
   N1 = floor(N/2) - N2 - N3;
   N2 = N1+N2;
   N3 = N2+N3;
   x(1:N1) = rand(size(1:N1));
   x(1:N1) = x(1:N1)/4;
   x(N1+1:N2) = rand(size(N1+1:N2));
   x(N1+1:N2) = 0.25 + x(N1+1:N2)*0.25;
   x(N2+1:N3) = rand(size(N2+1:N3));
   x(N2+1:N3) = 0.5 + x(N2+1:N3)*0.25;
   x(N3+1:N) = rand(size(N3+1:N));
   x(N3+1:N) = 0.75 + x(N3+1:N)*0.25;
   x = sort(x);
case 5 % sort of inverse devil's staircase
   k = N/(N-1);
   x = (0:1);
   nn = length(x);
   while (nn < N)
      xeven = x;
      xodd = xeven(1:nn-1);
      xodd = xodd + rand(size(xodd)).*(xeven(2:nn)-xeven(1:nn-1))/k;
      clear x
      nn = 2*nn-1;
      x(1:2:nn) = xeven;
      x(2:2:nn) = xodd;
      xeven = x;
      xodd = xeven(2:nn);
      xodd = xodd - rand(size(xodd)).*(xeven(2:nn)-xeven(1:nn-1))/k;
      clear x
      nn = 2*nn-1;
      x(1:2:nn) = xeven;
      x(2:2:nn) = xodd;
   end
   x = x(1:N)/x(N);
case 50 % non-random version of case 5
   k = (3/2)*N/(N-1);
   % k = 3*N/(N-1);
   x = (0:1);
   nn = length(x);
   while (nn < N)
      xeven = x;
      xodd = xeven(1:nn-1) + (xeven(2:nn)-xeven(1:nn-1))/k;
      clear x
      nn = 2*nn-1;
      x(1:2:nn) = xeven;
      x(2:2:nn) = xodd;
      xeven = x;
      xodd = xeven(2:nn) - (xeven(2:nn)-xeven(1:nn-1))/k;
      clear x
      nn = 2*nn-1;
      x(1:2:nn) = xeven;
      x(2:2:nn) = xodd;
   end
   x = x(1:N)/x(N);
case 8
   for (i = 1:9) x(i) = (i-1)/8; end
   M = 9;
   while (N - M > 3)
      for (i = 1:4) x(M+i) = (x(i) + x(i+1))/2; end
      M = M + 4;
      x = sort(x);
   end  
case 9
   Gap = 1000;
   k = div(N,4)*2+1;
   x = (1:N);
   x(k) = x(k) + Gap;
   x(k+1:N) = x(k+1:N) + 2*Gap;
   x = x/x(N);
case 16,
   M = ceil(log2(N));
   x = [0 1];  nj = 2;
   for j=1:M,
      u = rand(1,nj-1);
      xe = x; xo = xe(1:nj-1).*u+xe(2:nj).*(1-u);
      nj = 2*nj-1; x(1:2:nj) = xe; x(2:2:nj-1) = xo;
   end
   x = x(1:N)/x(N);
case 17, % uniform or given grid on coarse resolution, then refinement by
         % inserting all other points at once (not by multiresolution) 
   nL = floor(N/10);
   xL = NaN;
   randomrefinement = false;
   if length(gridoptions) > 0, 
      for o=1:length(gridoptions), 
         if isnumeric(gridoptions{o}) | islogical(gridoptions{o}), 
            optionsize = size(gridoptions{o});
            if optionsize == [1 1], 
               if ismember(gridoptions{o},[true false]),
                  randomrefinement = gridoptions{o};
               else
                  nL = gridoptions{o};
               end
            elseif min(optionsize) == 1,
               xL = gridoptions{o};
               nL = length(xL);
            end
         end
      end
   end
   if isnan(xL), xL = sort(rand(1,nL)); end
   xL = (xL-xL(1))/(xL(nL)-xL(1));
   knotsL = round((0:nL-1)*(N-1)/(nL-1)); 
   if randomrefinement == true,
      uu = staircase(0:N-1,knotsL,(0:nL-1)) + rand(1,N);
      uu(knotsL+1) = (0:nL-1); uu = sort(uu);
   else
      uu = (0:N-1)/(N-1)*(nL-1);
   end
   x = polyline(uu,(0:nL-1),xL);
case 610 % extreme case of grid with given homogeneity constant gamma>1; 
         % See Daubechies e.a., Regularity of irregular subdivision (Constr.
	 % Approx.)
   gamma = 5;
   if length(gridoptions) > 0, if isnumeric(gridoptions{1}), 
   if size(gridoptions{1}) == [1 1], gamma = gridoptions{1}; end, end, end
   x = makextrmhomogeneousgrid(N,gamma);
case 620 % non-homogeneous, but dyadically balanced grid with given balance
         % beta
   beta = 1/6;
   if length(gridoptions) > 0, if isnumeric(gridoptions{1}), 
   if size(gridoptions{1}) == [1 1], beta = gridoptions{1}; end, end, end
   % x = (0:1);
   x = (0:2)/2;
   nn = length(x);
   while nn < N,
      xx(1:2:2*nn-1) = x;
      xx(2:2:2*nn-1) = beta*x(1:nn-1) + (1-beta)*x(2:nn);
      nn = length(xx);
      x = xx;
   end
   x = x(1:N)/x(N);
case {666,6660,6661} % devil's staircase, cantor function
   Gap = 1/3;
   if gridID > 1000, Gap = 1/10; end
   if length(gridoptions) > 0, if isnumeric(gridoptions{1}),
   if size(gridoptions{1}) == [1 1], Gap = gridoptions{1}; end, end, end
   x = makecantorgrid(N,Gap,true,realrandom);
otherwise
  x = (1:N)/N;
end
x = a+x*(b-a);
