
function y = IWT2G1D(x,varargin);

%  IWT2G1D -- ThreshLab/ThreshLab2/Irregular/1D/ 2nd generation inv.wav.transf.
%   Usage
%     y = IWT2G1D(x,w,method,dualpar,primalpar,nlevels);
%     y = FWT2G1D(w,method,dualpar,primalpar,nlevels);
%   Inputs
%     x         ordered real vector, length n; covariate values
%     w         real vector or matrix, size n x m or m x n; wavelet coeff.
%     y         real vector or matrix, size n x m or m x n; response values
%     method    char; describing the refinement method (as in FWT2G1D)
%     dualpar   integer, matrix or cell array; parameters of refinement method
%     primalpar integer, integer vector, cell, char; parameters of detail basis
%     nlevels   integer; number of resolution levels
%   Outputs
%     y       vector of size(w) containing response values
%   Description
%     Fast inverse 2nd generation wavelet transform on irregular point sets
%     Applies repeated lifting filter bank. (More info in help FWT2G1D)
%   Note
%   See also
%     help FWT2G1D (forward transform)

% Since we have to run FWT2G1D for the construction of the lifting steps and
% the boundary refinements, we use FWT2G1D for the parsing of the input as
% well.
infofromFWT = FWT2G1D(x,varargin{:},'infoforIWT','extendwithzeros');
w = infofromFWT{1}; % the routine FWT2G1D has identified the wavelet
                    % coefficient vector (falsely assuming it were the fine
                    % scaling coefficients in a forward transform)
                    % and it has added zeros in front and at the end if
                    % necessary
splits = infofromFWT{2};
liftedrefinements = infofromFWT{3};
boundaryrefinements = infofromFWT{4};
finalupdates = infofromFWT{5};
dualtransform = infofromFWT{6};

transposex = false;
if size(x,2) > size(x,1),
   x = x'; transposex = true;
end
n = length(x);
transposew = false;
if size(w,1) < n,
   w = w'; transposew = true;
end
ny = size(w,2);
nn = size(w,1);

M = length(liftedrefinements);
y = w;
for j = M:-1:1,
   % get partitioning and boundaries at current scale
   e = splits{j,1}; o = splits{j,2};
   ne = length(e); no = length(o); nf = ne+no;
   s = y(1:ne,1:ny); d = w(ne+1:ne+no,1:ny);
   % UPDATE for wavelet functions
   U = finalupdates{j};
   if ~isnan(U), s = s - U*d; end
   % perform LIFTING STEPS for broken scaling functions
   UP = liftedrefinements{j}; Hbb = boundaryrefinements{j};
   y(1:nf,1:ny) = IWT1stepscale(s,d,UP,Hbb,dualtransform);
end

if transposew, y = y'; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = IWT1stepscale(s,d,UP,Hboundary,dualtransform)

ny = size(s,2); ne = size(s,1); no = size(d,1); nf = ne+no;
if ~isempty(Hboundary),
   iif = Hboundary{1}; bf = Hboundary{2};
   iic = Hboundary{3}; bc = Hboundary{4};
   Hbb = Hboundary{5}; Hib = Hboundary{6}; Hbi = Hboundary{7};
   e = Hboundary{8};
else
   e = (1:2:ne+no);
end
% Hbi is supposed to be zero, meaning that the refinement of the interior basis
% proceeds without intervention from the boundary (i.e., the interior basis is
% refinable on its own.)
if dualtransform == true, UP = duallifting(UP); end
if isempty(Hboundary), sii = s; else sii = s(iic,1:ny); end
yiif = invliftingsteps(sii,d,UP);
if isempty(Hboundary),
   y = yiif;
else
   y = zeros(nf,ny);
   if dualtransform == true,
      y(iif,1:ny) = yiif; y(bf,1:ny) = (Hbb')\Hib'*yiif;
   else
      y(iif,1:ny) = yiif+Hib*s(bc,1:ny);
      y(bf,1:ny) = Hbb*s(bc,1:ny);
   end
end
