function odds = findoddsinfinestgrid(odds,N,fromfinest);

% findoddsinfinestgrid -- book-keeping for refinement/coarsening
%  Usage
%    odds = findoddsinfinestgrid(odds,N,fromfinest);
%  Inputs
%    odds     array or cell array of indices
%    N        size of finest scale grid
%    fromfinest (optional) boolean/logical variable (default false)
%  Outputs
%    odds     array or cell array of indices; finest levels FIRST
%  Description
%    The multiresolution analysis of data on irregular grids can be represented
%    by a multiscale grid. This multiscale grid is uniquely defined by the
%    order in which points are added or removed to move from coarse to fine or
%    vice versa. The order of refinement in its turn can be represented in at
%    least two ways. This routine maps one way onto the other and vice versa.
%    Suppose odds is a cell array whose entries consist of indices into the
%    finest scale grid x (x itself not being used in this routine). Then
%    each entry in the (cell) array corresponds to one scale: x(odds{j}) are 
%    the points that are taken out to create a coarser grid from the grid at
%    scale j+1. 
%    The alternative way to represent the multiscale grid is to keep track of
%    the indices into the remaining points at a given level (so NOT an index
%    into the original points)
%    The boolean "fromfinest" specifies whether we go from indices into
%    finest grid to intermediate (coarser) grid or the other way around
%  See also
%    help RWT_2GHtPU
%    help IRT_2GHPU

if nargin < 3, fromfinest = false; end

M = length(odds);
evensj = (1:N);
if fromfinest == true,
   if isnumeric(odds), % one-coefficient-at-a-time refinement
      for j = 1:M,
         oddj = odds(j);
         o = find(evensj==oddj);
         evensj(o) = 0;
         evensj = evensj(find(evensj));
         odds(j) = o;
      end
   elseif iscell(odds),
      allnum = true; 
      for j=1:M, allnum = (allnum & isnumeric(odds{j})); end
      if allnum == true,
         for j = 1:M,
            oddsj = odds{j};
            o = find(ismember(evensj,oddsj));
            evensj(o) = 0;
            evensj = evensj(find(evensj));
            odds{j} = o;
         end
      end
   end
else
   evens = (1:N);
   % find absolute indices of odds-list (i.e. indices into finest scale grid x)
   if isnumeric(odds), 
      for j=1:M,
         Ne = length(evens);
         o = odds(j);
         e = setdiff((1:Ne),o);
         odds{j} = evens(o);
         evens = evens(e);
      end
   elseif iscell(odds),
      allnum = true;
      for j=1:M, allnum = (allnum & isnumeric(odds{j})); end
      if allnum == true,
         for j=1:M,
            Ne = length(evens);
            o = odds{j};
            e = setdiff((1:Ne),o);
            odds{j} = evens(o);
            evens = evens(e);
         end
      end
   end
end
