
function P = interpolynomialprediction(xeven,yeven,xodd,nt)

% interpolynomialprediction - ThreshLab2/Irregular/1D/ --- 
%                        interpolating or least squares polynomial prediction
% lifting step
%   Usage
%      yodd = interpolynomialprediction(xeven,yeven,xodd,nt);
%      P    = interpolynomialprediction(xeven,I,xodd,nt);
%   Inputs
%      xeven  x-coordinates of even indexed locations
%      yeven  y-values in even indexed locations
%      I      matrix (e.g., identity matrix of size = length(xeven) )
%      xodd   x-coordinates of odd indexed locations
%      nt     number of dual vanishing moments (= order of polynomial
%             prediction = degree of polynomial plus one, i.e.: 
%             2 -> linear; 4 -> cubic, etc.)
%             nt can also be a PAIR of integers (see Description below)
%   Outputs
%      yodd   predicted values in odd indexed locations
%      P      for I = eye(length(xeven)), P is the prediction operation matrix
%   Description
%      This routine evaluates an interpolating polynomial in each point of xodd.
%      The interpolating polynomial may be different for every point in xodd.
%      if nt == 2, this routine coincides with polyline interpolation
%                  except for points outside [min(xeven),max(xeven)]
%      if nt is a PAIR of integers, then the first component is the degree of
%                 the polynomial prediction, while the second component stands
%                 for the additional number of neighbours taken into account
%                 for a LEAST SQUARES polynomial prediction
%      If yeven is a matrix (e.g., yeven=I), then polynomial prediction is
%      performed on all ROWS of yeven if xeven is a row vector
%                       COLUMNS of yeven if xeven is a column vector
%   See Also
%      help interpolationneville
%      help polyline
%      help averagepolynomialprediction


Ne = length(xeven);
Noo = length(xodd);
[xodd oo] = sort(xodd);
% we assume xeven and xodd are row vectors. If xodd turns out to be column
% vector, then a final tranpose is applied to P
Ptranspose = false;
if size(xeven,1) > 1, xeven = xeven'; Ptranspose = true; end
if size(xodd,1) > 1, xodd = xodd'; Ptranspose = true; end
[r s] = size(yeven);
if s ~= Ne, 
   yeven = yeven';
   [r s] = size(yeven);
   if s ~= Ne, error('dimension mismatch inputs xeven & yeven'), end
end
% now s == Ne

nut = nt(1);
if length(nt)>1, nut = sum(nt); nt = nt(1); end
% nut (nu-tilde) is the number of parameters to be defined in the prediction
% nt is the number of dual vanishing moment conditions
% nut-nt are other parameters to be completed by least squares conditions

P = zeros(r,Noo);
if (Ne < nt) 

  nt = Ne;
  P = interpolationneville(xodd,xeven,yeven);

elseif nt>0,

  lefteven = closestleftinvector(xodd,xeven);
  No = length(xodd); % maximum value is Ne+1


  nl = ceil(nut/2); nr = nut-nl;
  % if nut is odd, then the prediction will be based on nl even points on the
  % left and nl-1 on the right. As the first point in a split is always even,
  % there is a slight tendency to have more points on the left available
  for e = 1:nl-1, o = (lefteven==e); if any(o), o = find(o);
    if nut > nt,
       P(1:r,o) = leastsquarespolynomial(xodd(o),xeven(1:nt),...
                                         yeven(1:r,1:nt),nt-1);
    else
       P(1:r,o) = interpolationneville(xodd(o),xeven(1:nt),yeven(1:r,1:nt));
    end
  end, end
  for e = nl:Ne-nr, o = (lefteven==e); if any(o), o = find(o);
    if nut > nt,
       P(1:r,o) = leastsquarespolynomial(xodd(o),xeven(e-nl+1:e+nr),...
                                         yeven(1:r,e-nl+1:e+nr),nt-1);
    else
       P(1:r,o) = interpolationneville(xodd(o),xeven(e-nl+1:e+nr),...
                                              yeven(1:r,e-nl+1:e+nr));
    end
  end, end
  for e = Ne-nr+1:No, o = (lefteven==e); if any(o), o = find(o);
    if nut > nt,
       P(1:r,o) = leastsquarespolynomial(xodd(o),xeven(Ne-nt+1:Ne),...
                                         yeven(1:r,Ne-nt+1:Ne),nt-1);
    else
       P(1:r,o) = interpolationneville(xodd(o),xeven(Ne-nt+1:Ne),...
                                       yeven(1:r,Ne-nt+1:Ne));
    end
  end, end

end
P(1:r,oo) = P;
if Ptranspose==true, P = P'; end
