function y = invliftingsteps(s,d,UP,e);

% invliftingsteps - ThreshLab2/Irregular/1D/ --- inverse filterbank transform 
%                   using given lifting steps 
% Usage
%   y = invliftingsteps(x,UP);
%   y = invliftingsteps(s,d,UP);
% Input
%   x     vector or matrix containing scaling and detail coefficients
%   s     vector or matrix containing scaling coefficients
%   d     vector or matrix containing detail coefficients
%   UP    cell array of matrices
% Output
%   y     vector or matrix
% Note
%   (1)
%   The current implemention assumes that the last lifting step is a rescaling
%   of the evens and the one but last step is a prediction of the odds
%   (2)
%   The implementation slightly deviates from the paper on Nonequispaced
%   B-spline wavelets in the Theorem (wavelet transform from lifting
%   factoring). This implementation adopts the defintion P = H_{j,o} for
%   the final prediction step, while the paper uses P = H_{j,o}/D.
% See also
%   help frwliftingsteps
%   help refinementfromliftingsteps (for nearly the same routine)

if iscell(d),
   if nargin>2, e = UP; else, e = NaN; end
   x = s; UP = d;
else
   x = NaN;
   if nargin<4, e = NaN; end
end 

transposex = false;
nUP = length(UP)-1;
noP = false; DisI = false;
if nUP>-1, D = UP{nUP+1}; else, DisI = true; end
if nUP>0, P = UP{nUP}; else, noP = true; end
if noP==false,
   [no, ne] = size(P);
   n = ne+no;
   if isnan(e(1,1)), e = (1:2:n); end
   if length(e) ~= ne,
      error('dimension mismatch prediction matrix and vector of even indices')
   end
   if ~isnan(x(1,1)),
      sizex = size(x); 
      if sizex(1) ~= n, x = x'; transposex = true; sizex = size(x); end
      if sizex(1) ~= n, error('dimension mismatch UP and x'); end
      m = sizex(2);
   end
elseif ~isnan(x(1,1)),
   [n m] = size(x); 
   if isnan(e(1,1)), e = (1:2:n); end
   ne = length(e);
   no = n-ne;
   P = zeros(no,ne);
else
   n = NaN;
end
if isnan(x(1,1)),
   ne = size(s,1); no = size(d,1);
   if isnan(n), 
      if size(s,2)~=size(d,2),
         s=s'; d=d'; ne = size(s,1); no = size(d,1); transposex = true;
      end
      n = no+ne;
   else
      if n~=no+ne,
         s=s'; d=d'; ne = size(s,1); no = size(d,1); transposex = true;
      end
      if n~=no+ne,
         error('dimension mismatch prediction matrix and vectors s and d')
      end
   end
   if isnan(e(1,1)), e = (1:2:n); end
   m = size(s,2);
   x = zeros(n,m);
   x(e,1:m) = s; o = setdiff(1:n,e); x(o,1:m) = d;
else
   o = setdiff(1:n,e);
end
if DisI == true, D = eye(ne); end
if length(o) ~= no,
   error('dimension mismatch prediction matrix and vector of odd indices')
end

y = x;
% Next line deviates from Theorem on wavelet transform from liftig
% factoring in paper on Nonequispaced B-spline wavelets. This
% implementation defines the final prediction in the lifting factoring as
% P = H_{j,o}, whereas the paper adopts P = H_{j,o}/D
y(o,1:m) = y(o,1:m) + P*y(e,1:m);
y(e,1:m) = D*y(e,1:m);

thisstepisprediction = false;
for s = nUP-1:-1:1,
   if thisstepisprediction,
      P = UP{s};
      y(o,1:m) = y(o,1:m) + P*y(e,1:m);
      thisstepisprediction = false;
   else
      U = UP{s};
      y(e,1:m) = y(e,1:m) - U*y(o,1:m);
      thisstepisprediction = true;
   end
end

if transposex == true, y = y'; end
