function [H, G] = refinementfromliftingsteps(UP,e);

% refinementfromliftingsteps - ThreshLab2/Irregular/1D/ --- 
%          finds refinement matrices for two-scale equation from lifting
%          decomposition
% Usage
%   [H, G] = refinementfromliftingsteps(UP);
%   [H, G] = refinementfromliftingsteps(UP,e);
% Input
%   UP    cell array of matrices, with lifting steps ordered for application
%         from right to left (matrix product) in the refinement
%         (=reconstruction or inverse wavelet transform)
% Output
%   H     matrix in refinement (2-scale) equation: PhiJ*H = PhiL
%   G     matrix in wavelet equation:              PhiJ*G = PsiL
% Description
%   PhiJ   is the fine scaling basis
%   PhiL   is the coarse scaling basis
%   PsiL   is the wavelet basis
% Notes
%   (1) This routine asumes a partitioning into evens and odds. 
%       This means: critical downsampling, no overlap, 
%                   no nondecimated transforms, no cycle-spinning, no redundancy
%   (2) This routine is equivalent to
%       nUP = length(UP)-1;
%       P = UP{nUP};
%       [no, ne] = size(P);
%       I = eye(ne+no);
%       W = invliftingsteps(I,UP);
%       [even odd] = splitevenodd(ne+no);
%       H = W(:,even); G = W(:,odd);
%   (3) The implemention assumes that the first lifting step on the coarse
%   scale data is a rescaling of the evens and the next step is a prediction of
%   the odds. This means: the last step in the forward transform is a rescaling
%   and the one but last step in the forward transform is a prediction. The
%   first lifting step in the reconstruction (the rescaling), is the rightmost
%   in the matrix-product, so that is D = UP{nUP+1}
%   If the one but last step in the reconstruction is an update rather than a 
%   prediction, then insert a new void prediction:
%       nUP = length(UP);
%       UP = {UP{1:nUP},UP{nUP}};
%       U = UP{nUP-1};
%       [ne, no] = size(U);
%       UP{nUP} = zeros(no,ne);
%   (4) The implementation slightly deviates from the paper on Nonequispaced
%   B-spline wavelets in the Theorem (wavelet transform from lifting
%   factoring). This implementation adopts the defintion P = H_{j,o} for
%   the final prediction step, while the paper uses P = H_{j,o}/D.
%   (5) Use UPt = duallifting(UP); [Ht, Gt] = refinementfromliftingsteps(UPt);
%       to find the dual matrices without explicit matrix inversion.
%   (6) If the partitioning even-odd is not alternating and if parameter e is
%   not given, the routine finds the number of evens and arranges H as [He;Ho] 
%   (as no it has no information on the precise position of the evens)
% See also
%   help liftingstepsBspline
%   help invliftingsteps
%   help liftingstepsfromrefinement

nUP = length(UP)-1;
D = UP{nUP+1};
P = UP{nUP};
[no, ne] = size(P);
n = ne+no;
if nargin < 2, 
   if abs(ne-no)<2,
      e = (1:2:n); 
   else
      e = (1:ne);
   end
end
if length(e) ~= ne,
   error('dimension mismatch prediction matrix and vector of even indices')
end
o = (1:n); o(e) = 0; o = find(o);
if length(o) ~= no,
   error('dimension mismatch prediction matrix and vector of odd indices')
end

H = sparse(zeros(n,ne));
H(e,1:ne) = D;
H(o,1:ne) = P;

G = sparse(zeros(n,no));
G(o,1:no) = sparse(eye(no));

thisstepisprediction = false;
for s = nUP-1:-1:1,
   % add lifting step on fine scale side
   if thisstepisprediction,
      P = UP{s};
      H(o,1:ne) = H(o,1:ne) + P*H(e,1:ne);
      G(o,1:no) = G(o,1:no) + P*G(e,1:no); 
      thisstepisprediction = false;
   else
      U = UP{s};
      H(e,1:ne) = H(e,1:ne) - U*H(o,1:ne);
      G(e,1:no) = G(e,1:no) - U*G(o,1:no);
      thisstepisprediction = true;
   end
end
