function [U, We, Wte] = updateLSapprxprimmom(UP,nu,Me,Mo,We,Wte,dd,solver)

% updateLSapprxprimmom -  ThreshLab2/Irregular/1D/ --- update or primal lifting
% step, designed to make transform as close as possible to being orthogonal
%   Usage
%      [U, We, Wte] = updateLSapprxprimmom(UP,nu,Me,Mo,We,Wte);
%      [U, We, Wte] = updateLSapprxprimmom(UP,nu,NaN,NaN,We,Wte);
%      [U, We, Wte] = updateLSapprxprimmom(UP,nu,Me,U0,We,Wte);
%   Inputs
%      UP  cell array of matrices, the lifting steps
%          alternatively, UP is replaced by {'filterbank',H,G,Ht,Gt}
%      nu  integer; number of free parameters; i.e., nonzero diagonals in U
%      Me  matrix; column p contains integral of x^(p-1)*phi_e(x), where 
%                  phi_e(x): basis function on even branch of lifting scheme
%                  *after* application of all steps in UP
%      Mo  matrix; column p contains integral of x^(p-1)*phi_o(x), where 
%                  phi_o(x): basis function on odd branch of lifting scheme
%                  *after* application of all steps in UP
%      U0  matrix of size(U), bandmatrix solving the moment conditions, i.e.,
%                  Mo = U0'*Me (can be found using updateprimalmoments)
%      We  matrix; submatrix of inverse wavelet transform corresponding to the
%                  current scaling coefficients for the reconstruction of the
%                  finest scaling coefficients (y = We*s)
%      Wte matrix; submatrix of forward wavelet transform mapping fine scaling
%                  coefficients onto current scaling coefficients (s = Wte'*y)
%      dd (optional) vector or matrix of indices, with several possible
%                    meanings
%                    (1) integer vector of length nu, containing list of
%                        nonzero diagonals: dd(d) is the difference k-l for an
%                        element U(k,l) on the d-th diagonal
%                    (2) nu x No matrix of integers, containing for each
%                        column of U the list of nonzero rows
%   Outputs
%      U   matrix: update step
%      We  matrix: coarser scale version of input We
%      Wte matrix: coarser scale version of input Wte
%   Description
%      This routine computes the update matrix with nu nonzero diagonals that
%      minimizes the Frobenius norm of the projection (We*Wte) onto the coarse
%      scale, while anihilating the moments of the detail (wavelet) basis
%      functions. 
%      nu: total number of degrees of freedom
%      n : number of primal vanishing moments (n must not be larger than nu)
%          n = size(Me,2) = size(Mo,2)
%          Put Me=NaN for n=0
%   Reference
%      This routine solves Eq.(70) in the paper 
%      Non-equispaced B-spline wavelets (arXiv:1502.01533; version 5 Feb 2015)
%   Note
%      When nu = n, then use updateprimalmoments
%   See also
%      help updateprimalmoments

UP1 = UP{1};
if ischar(UP1),  % UP contains filterbank H,G0,Ht0,Gt
   nUP = length(UP)-1;
   UP = UP(2:nUP+1);
   H = UP{1}; G0 = UP{2};
   if nUP < 4, 
      nH = size(H,2); nG0 = size(G0,2);
      Ht0Gt = inv([H G0]);
      Ht0 = Ht0Gt(1:nH,1:nH+nG0)';
      Gt = Ht0Gt(nH+1:nH+nG0,1:nH+nG0)';
   else
      Ht0 = UP{3}; Gt = UP{4};
   end
else
   UPt = duallifting(UP);
   [Ht0 Gt] = refinementfromliftingsteps(UPt);
   [H G0] = refinementfromliftingsteps(UP);
end
if nargin < 4,
   Mf = Me;
   Me = H'*Mf;
   Mo = G0'*Mf;
end
if isnan(Me), n = 0; else, n = size(Me,2); end

Ne = size(H,2);
No = size(G0,2);
N = Ne+No;
nu = min(nu,N);

% If Mo has the size of U, then Mo carries the values of U0, satisfying 
% Mo = U0'*Me. We then impose U'*Me = 0, so that (U+U0)'*Me = Mo
if all(size(Mo) == [Ne No]) | all(size(Mo) == [No Ne]),
   U0 = Mo;
   Mo = zeros(No,n);
else
   U0 = zeros(Ne,No);
end

if nargin < 8,
   solver = 'LU';
end
% specification of the band structure
defaultdd = floor((1:nu)/2).*(-1).^((1:nu)-2*floor((1:nu)/2));
if nargin < 7,
   dd = defaultdd;
elseif ischar(dd(1)),
   solver = dd;
   dd = defaultdd;
elseif min(size(dd))==1,
   nu = min(nu,length(dd));
   dd = dd(1:nu);
end
% make sure that every column of dd lists the nonzero rows in the corresponding
% column of U. The design of U is thus based on conditions formulated in the
% odds (although U has an effect on the even coefficients, its design aims at
% properties of the odd basis functions)
if min(size(dd))==1,
   dd = column(dd)*ones(1,No)+ones(nu,1)*(1:No);
end

%%%%%%%%%%%%% system setup %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
WeH = We*H;
WteHt0 = Wte*Ht0;
WteGt = Wte*Gt;

Fe = WeH'*WeH;
Co = WteGt'*WteGt;
Be = -WteHt0'*WteGt-U0*Co;
Re = Fe*Be;
% We have to find a band matrix U so that the matrix expression 
% Fe*U*Co = Fe*Be = Re, holds,
% but only in the components (r,s) where U(r,s) is nonzero

%%%%%%%%%%%%% system solving %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nu > Ne, U = Be/Co; else,
A = [];
b = [];
for e = 1:nu, % consider the e-th diagonal (r,s) with s = r-dd(e) in the
              % system of equations: partial derivative for U(r,s) = 0
              % This system amounts to Fe*U*Co = R
   s = (1:No); r = dd(e,s); i = find(r>0 & r<Ne+1); r = r(i); s = s(i);
   nUe = length(i);
    
   Ae = zeros(nUe,0);
   for d = 1:nu,
      l = (1:No); k = dd(d,l); j = find(k>0 & k<Ne+1); k = k(j); l = l(j);
      Ae = [Ae Fe(r,k).*Co(l,s)'];
      % note that in general C(l,s)' is not the same as C(s,l)
      % (In this case, it is, because C is symmetric)
   end
   for m = 0:n-1,
      Aem = zeros(nUe,No);
      Aem(1:nUe,i) = -diag(Me(r,m+1));
      Ae = [Ae Aem];
   end
   A = [A; Ae];
   b = [b; diag(Re(r,s))];
end
% Next lines fill in the constraint equations Mo = U'*Me
for m=0:n-1, % for every column in Mo and Me 
   % Current implementation imposes one moment equation for every element in
   % Mo(1:No,m+1), which amounts to No*n moment equations in total.
   % It should be noted that No*n is more than the number of free parameters
   % that can be filled in on n diagonals (unless n is 1 or n = 2 and No=Ne-1)
   % cfr: updateprimalmoments.m, where the higher moment conditions are omitted
   % for the basis functions near the boundary.
   % We could restrict the number of moment conditions, as in
   % Am = zeros(NUd(m+1),0);
   % but it is unlcear which diagonals should then be counted for moment
   % equations, and which for minimum Frobenius norm. (Diagonals are of unequal
   % length). Therefore we impose all No*n moment equations.
   % Note that when nu-n = 0, then the number of equations is larger than the
   % number of nonzeros in U, and so the system is overdetermined. Therefore,
   % the outcome of updateLSapprxprimmom.m may have no vanishing moments at
   % all. Use updateprimalmoments.m for the case where nu=n
   Am = zeros(No,0);
   for d = 1:nu, % i.e., for every diagonal of U
      l = (1:No); k = dd(d,l); i = find(k>0 & k<Ne+1); k = k(i); l = l(i);
      nUd = length(i);
      Amd = zeros(No,nUd);
      Amd(i,1:nUd) = diag(Me(k,m+1)); % rows in Me -> columns in U'
      Am = [Am Amd];
   end
   % The zero submatrix in the following line corresponds to the Lagrange
   % multipliers: they do not have any coefficient in the constraint equations
   A = [A; Am zeros(No,n*No)];
   b = [b; Mo(1:No,m+1)]; % the corresponding right hand side is a column of Mo
end

switch solver,
case {'krylov','Krylov'},
   x = minxAxminbxkrylov(A,b);
% case {'Gauss-Seidel','GS'},
%  x = solveGS(A,b,0);
otherwise, % case {'LU','lu','Gauss','gauss'},
   x = A\b;
end
U = zeros(Ne,No);
% Ucol = zeros(Ne*No,1);
kl = 0;
for d = 1:nu,
   l = (1:No); k = dd(d,l); i = find(k>0 & k<Ne+1); k = k(i); l = l(i);
   nUd = length(k);
   if nUd>0,
      kl = kl+(1:nUd);
      % We now use the Matlab implementation of matrix indexation, using a
      % single index, arranged column by column. 
      % If this doesn't work in other software, then first introduce a column
      % vector
      % Ucol = zeros(Ne*No,1); 
      % fill it up with the entries as done here and then reshape it to U
      U(k+(l-1)*Ne) = x(kl);
      kl = kl(nUd);
   end
end

end % if nu > N-1, U = B/C; else,

U = U+U0;
We = WeH;
Wte = WteHt0 + WteGt*U';
