
function U = updateprimalmoments(xodd,yodd,xeven,Modd,Meven,dd);

% updateprimalmoments - ThreshLab2/Irregular/1D/ --- update or primal lifting
% step, designed for vanishing moments of primal (=synthesis) wavelet basis
%   Usage
%      yeven = updateprimalmoments(xodd,yodd,xeven,Modd,Meven);
%      U     = updateprimalmoments(xodd,I,xeven,Modd,Meven);
%      U     = updateprimalmoments(xodd,xeven,Modd,Meven);
%      U     = updateprimalmoments(xodd,xeven,Modd,Meven,dd);
%      U     = updateprimalmoments(Modd,Meven,dd);
%      U     = updateprimalmoments(Modd,Meven);
%   Inputs
%      xodd    x-coordinates of odd observations/odd fine scale coefficients
%      yodd    y-values in odd indexed locations
%      I       matrix (e.g., identity matrix of size = length(xodd) )
%      xeven   ``even'' indexed locations, i.e., locations that proceed to
%              coarse scale
%      Modd    matrix whose p-th column is (p-1)st moment of the current odd
%              scaling functions (first column thus contains integrals of
%              scaling functions). These moments are to be annihilated by
%              this routine, thereby turning the odd scaling functions into 
%              wavelet functions.
%      Meven   matrix whose p-th column is (p-1)st moment of the even scaling
%              functions (These scaling functions and their moments are left
%              untouched by this update operation)
%      dd      vector or matrix of indices, with three possible interpretations
%              (1) integer, containing the number of nonzero diagonals in U
%                  (the number of degrees of freedom)
%              (2) integer vector of length nmom, containing list of nonzero
%                  diagonals: dd(d) is the difference k-l for an element U(k,l)
%                  on the d-th diagonal
%                  (nmom being the number of vanishing moments)
%              (3) nmom x No matrix of integers, containing for each column of
%                  U: the list of nonzero rows
%   Outputs
%      yeven  values to be added to in even indexed coefficients
%      U      for I = eye(length(xeven)), U is the update operation matrix
%   Description
%      This routine computes the update that annihilates the moments of the
%      primal (=synthesis) wavelet (= detail) basis functions in a lifting 
%      scheme. The number of moments that are annihilated (i.e., the number
%      of primal vanishing moments) equals the number of columns in Mfine and
%      Mcoarse. The columns of Mfine and Mcoarse provide sets of linear 
%      equations for the columns of U, hence, the number of nonzero elements in
%      each column of U is bounded by the number of columns in Mfine and
%      Mcoarse
%      The moment anihilation equation is
%
%         0 = Modd(:,p) - U'*Meven(:,p)
%
%      for p = 1,...,nmom  where nmom is the number of moments to be
%      annihilated (the number of vanishing moments)
%      This has to be interpreted as a set of n equations for every row (o) of
%      Modd. The unknowns of each set are the nonzero elements in the o-th
%      column of U
%      This routine combines several implementations, depending on the
%      parameters passed by the calling algorithm
%
%   See Also
%      help interpolynomialprediction
%      help updateLSapprxprimmom

switch nargin,
case 2,
   Modd = xodd; Meven = yodd; n = size(Modd,1)+size(Meven,1);
   [xeven xodd] = splitevenodd(n); yodd = NaN; dd = NaN;
case 3,
   Modd = xodd; Meven = yodd; dd = xeven; xeven = NaN; xodd = NaN; yodd = NaN;
case 4,
   Meven = Modd; Modd = xeven; xeven = yodd; yodd = NaN; dd = NaN;
case 5,
   if min(size(Meven))~=min(size(Modd)) | max(size(Meven))~=max(size(xeven)),
   % if Meven does not fit with Modd and xeven, the call
   % U = updateprimalmoments(xodd,yodd,xeven,Modd,Meven);
   % should be reinterpreted as
   % U = updateprimalmoments(xodd,xeven,Modd,Meven,dd);
      dd = Meven; Meven = Modd; Modd = xeven; xeven = yodd; yodd = NaN;
   else,
      dd = NaN;
   end
end
nmom = size(Modd,2); % number scaling moments, max.number of vanishing moments
if size(Meven,2) ~= nmom, error('dimension mismatch in Modd and Meven'), end
Ne = size(Meven,1); No = size(Modd,1);

if isnan(dd),
   if length(xeven) ~= Ne, 
      error('dimension mismatch in Meven and xeven')
   end
   if length(xodd) ~= No,
      error('dimension mismatch in Modd and xodd')
   end
   dd = nmom;
end

if Ne==1 & No==1 & isnan(Meven(1,1)),
   % This is de case where xeven and xodd are given, but Meven and Modd are
   % Nan. Then make sure U = zeros(Ne,No)
   Ne = length(xeven); No = length(xodd);
end

if min(size(dd))==1,
   if max(size(dd))<3, % i.e., dd=dof (degrees of freedom)
      dof = min(nmom,sum(dd)); dof1 = min(nmom,dd(1));
      dd = bandliftingstep(xeven,xodd,dof);
      % if dof is odd, then one more even on the left is involved in the update
      % than on the right (cfr. interpolynomialprediction)
   else
      dd = column(dd)*ones(1,No)+ones(nmom,1)*(1:No);
   end
end
dof = size(dd,1);

U = zeros(Ne,No);
if ~isnan(Meven(1,1)),
   for o = 1:No,
      e = dd(1:dof,o); e = e((e>0)&(e<Ne+1));
      A = Meven(e,1:dof1)'; a = Modd(o,1:dof1)';
      % B = Meven(e,dof1+1:nmom)'; b = Modd(o,dof1+1:nmom)';
      % the version above may lead to singular normal equations, if B has too
      % few equations. Therefore, include all moments, also those included in A
      if nmom>dof1,
         B = Meven(e,1:nmom)'; b = Modd(o,1:nmom)';
      else % avoiding singular, unnecessary B
         B = zeros(0,length(e)); b = zeros(length(e),0);
      end
      % Let U(e,o) minimize norm(B*u-b) subject to A*u=a
      U(e,o) = constrLS(B,b,A,a);
      % U(e,o) = (Meven(e,1:nmom)')\(Modd(o,1:nmom)');
   end
else
   U = NaN;
end
U = sparse(U);
if ~isnan(yodd),
   if size(yodd,1) ~= size(U,2), yodd = yodd'; end
   U = U*yodd;
end
