function w = FWT2G2Dsquared(varargin);

%  FWT2G2Dsquared -- square 2nd gen. frwd. wav. transf. applied to 2D data; 
%   Usage
%     w = FWT2G2Dsquared(x,y,pt,p,M,<other arguments>);
%     w = FWT2G2Dsquared(x1,x2,y,pt,p,M,<other arguments>);
%     w = FWT2G2Dsquared(y,pt,p,M,<other arguments>);
%   Input
%     x      cell of two vectors: x = {x1,x2}
%     x1     vector of first coordinate in tensor product (isotropic) grid
%     x2     vector of second coordinate in tensor product (isotropic) grid
%            default of x1 and x2 are (1:m) and (1:n) with [m n] = size(y)
%     y      matrix of observations on (x1,x2)
%            or vector of length(x1)*length(x2)
%            or matrix with vectors of length(x1)*length(x2)
%     pt     number of dual vanishing moments (default 1)
%     p      number of primal vanishing moments (default 1)
%     M      number of resolution levels (default 1)
%     'rect' (optional) activates rectangular (isometric) transform
%     <other arguments> are those of FWT2G1D
%   Output
%     w      matrix of size(y)
%   See also
%     help FWT2G1D    (1D version)
%     help IWT2G2Dsquared  (inverse transform; not yet implemented)

% Parsing varargin
nvarargin = length(varargin); par = ones(1,nvarargin);
k = 0;
y = NaN; ptpM = {1,1,1};
transform = 'squared';
i = 1; j = 1; x = {NaN,NaN};
reshapey = false;
while k < nvarargin, k=k+1; vark = varargin{k}; 
   if isnumeric(vark),
      [n m] = size(vark);
      if min(n,m) > 2, y = vark; par(k) = 0;
      elseif min(n,m) == 2 & i<2
         x{1} = vark(1); x{2} = vark(2); i=3; par(k) = 0;
      elseif max(n,m) > 2,
         if i<3, x{i} = vark; i = i+1; par(k) = 0; 
         else, n1 = length(x{1}); n2 = length(x{2});
            if n==n1*n2 | m==n1*n2, y = vark; par(k) = 0; end
         end
      elseif j<4, ptpM{j} = vark; j = j+1; par(k) = 0;
      end
   elseif ischar(vark), switch(vark)
      case {'rect','rectangular'}
         transform = 'rect';
      case {'squared'}
         transform = 'squared';
      end
   end
end
par = find(par);
pt = ptpM{1}; p = ptpM{2}; M = ptpM{3};
x1 = x{1}; x2 = x{2};
if isnan(x1), n1 = size(y,1); x1 = (1:n1)/n1; end
if isnan(x2), n2 = size(y,2); x2 = (1:n2)/n2; end
n1 = length(x1); n2 = length(x2); [n m] = size(y);
if n==n1*n2 | m==n1*n2, reshapey = true; end
if reshapey==true,
   y = reshape(y,n1,m*n2);
   cc = (0:m*n2-1); cc = cc-n2*floor(cc/n2)+1; c = find(cc<=n2);
else
   m = 1; n = n1*n2;
end
switch transform
case {'rect'},
   w1 = FWT2G1D(x1,y,pt,p,M,varargin(par));
   if reshapey==true & m>1,
      w1 = blocktranspose(w1,n1,n2);
   else
      w1 = w1';
   end
   w = FWT2G1D(x2,w1,pt,p,M,varargin(par));
   if reshapey==true & m>1,
      w = blocktranspose(w,n2,n1);
   else
      w = w';
   end
otherwise
   w = y;
   w2 = w;
   for j = 1:M,
      [w1, ne1, mom1, o1] = FWT2G1D(x1,w2,pt,p,1,varargin(par));
      if reshapey==true & m>1,
         w1 = blocktranspose(w1,n1,n2);
      else
         w1 = w1';
      end
      [w2, ne2, mom2, o2] = FWT2G1D(x2,w1,pt,p,1,varargin(par));
      if reshapey==true & m>1,
         w2 = blocktranspose(w2,n2,n1);
         w(1:n1,c) = w2;
      else
         w2 = w2';
         w(1:n1,1:n2) = w2;
      end
      o1 = o1{1}; o2 = o2{1};
      e1 = setdiff(1:n1,o1); x1 = x1(e1); n1 = length(e1);
      e2 = setdiff(1:n2,o2); x2 = x2(e2); n2 = length(e2);
      if reshapey==true & m>1,
         c = find(cc<=n2); w2 = w(1:n1,c);
      else
         w2 = w2(1:n1,1:n2);
      end
   end
end
if reshapey==true, w = reshape(w,n,m); end
