function [H o] = refinementbrokenbasis(LRknots,nutilde)

% refinementbrokenbasis - ThreshLab2/Irregular/BrokenbasisWT/ --- 
%                    even-odd refinement for piecewise smooth basis functions
% Usage
%    H = refinementbrokenbasis(LRknots)
%    H = refinementbrokenbasis(knots,Omega);
% Inputs
%    LRknots matrix of size [2*n x (n+nutilde-2)], output of brokenbasis
%    knots   vector of size n
%    Omega   cell array or matrix or integer, as for brokenbasis
% Outputs
%    H       matrix of size (n+nutilde-2) x (ne+nutilde-2)
%            where ne = ceil(n/2)
% Description
%    The input matrix is obtained as output in
%        [Phi aa LRknots] = brokenbasis(...)
%    Its elements are interpreted as:
%    LRknots(2l-1,k) has the left limit of the (nutilde-1) derivative of
%                    a piecewise smooth basis function phi_k(x) in x_l
%    LRnots(2l,k) has the right limit
%    Considering the row of functions Phi(x) = [phi_1(x) ... phi_k(x) ...],
%    this function constructs the matrix H such that
%    Phi_c(x) = Phi(x)*H is a row coarse scale basis functions phi_{c,l}(x)
%    for which the (nutilde-1) derivatives in the odd knots x_{2l+1} are
%    continuous (so that these odd knots become interior points)
% See also
%    help brokenbasis

if min(size(LRknots))==1, % LRknots is actually knots
   knots = LRknots;
   if nargin<2, nutilde = 1; end
   [PhiBBknots aa LRknots] = brokenbasis(knots,knots,nutilde);
end
n = size(LRknots,1)/2; % number of knots
nn = size(LRknots,2); % number of basis functions
nutilde = size(LRknots,2)-n+2; % degrees of freedom (e.g. vanishing moments)
nU = floor((nutilde+1)/4); % number of update steps
r = ceil(nutilde/2) - 2*nU;
rtilde = mod(nutilde,2);
pt = ceil(nutilde/2)-1;

[e o] = splitevenodd(n);
ne = length(e); no = length(o); ee = [(1:pt) pt+e (nn-pt+r+1:nn)]; oo = pt+o;

ne = length(e); no = length(o); nee = ne+nutilde-2;
L = LRknots(1:2:2*n,1:nn); % left limits in each knot
R = LRknots(2:2:2*n,1:nn); % right limits in each knot
Delta = R-L; 

Deltao = Delta(o,1:nn);
H = zeros(nn,nee);
for k=1:nutilde,
   ll = (k:2*k); l = ll(2:k+1);
   H(k,k) = 1-sum(H(k,1:k-1),2);
   b = -Deltao(1:k,k)*H(k,k);
   A = Deltao(1:k,k+1:2*k);
   H(l,k) = A\b;
end
nl = nutilde+1;
for k=nutilde+1:no % ?? nee-nutilde+1,
   ll = ll+2;
   l1 = ll(1:2); l = ll(3:nl);
   H(l1,k) = 1-sum(H(l1,1:k-1),2);
   b = -Deltao(k-nutilde:k,l1)*H(l1,k);
   A = Deltao(k-nutilde:k,l);
   H(l,k) = A\b;
end
for k=no+1:nee, % nee-nutilde+2:nee,
   ll = ll+2; nl = nl-1; ll = ll(1:nl);
   l1 = ll(1:2); l = ll(3:nl);
   H(l1,k) = 1-sum(H(l1,1:k-1),2);
   b = -Deltao(k-nutilde:no,l1)*H(l1,k);
   A = Deltao(k-nutilde:no,l);
   H(l,k) = A\b;
end
% H = sparsesolveAXisB(Deltao,O);
