
function [OmegaL Omega] = smoothsetmatrix(knots,Omega);

% smoothsetmatrix - ThreshLab2/Irregular/BrokenbasisWT/ --- construct
%                    matrix of knot values and derivatives of smooth functions
% Usage
%    Omega = smoothsetmatrix(Omega);
%    Omega = smoothsetmatrix(knots,Omega);
%    [OmegaL Omega] = smoothsetmatrix(...)
% Inputs
%    knots   real vector of length n, defining the knots
%    Omega   one of the following definitions of the set of smooth functions
%            (1) cell of (up to) nutilde-1 matrices of size (n x nutilde)
%                the k-th column of Omega{q} has the (q-1)st derivative of the
%                smooth function Omega_k(x) evaluated in knots
%                (missing derivatives are completed by numerical computation)
%            (2) cell of (up to) nutilde-1 matrices of size ((n-1) x nutilde)
%                defining values and derivative of locally specified smooth
%                functions, i.e., the values and derivatives in
%                sawtoothknots(x,knots)
%            (3) cell of length three, containing {OmegaL,Omega,'LR'} with
%                OmegaL and Omega the outputs from a previous run of this
%                routine. Nothing needs to be done.
%            (3) matrix of size (n x nutilde)
%            (4) matrix of size [nutilde*n] x nutilde
%                            or [nutilde*(n-1)] x nutilde
%                which will be delivered as output, unchanged
% Outputs
%    Omega   real matrix, size [nutilde*n] x nutilde
%                           or [nutilde*(n-1)] x nutilde
%            defining the set of smooth functions for use in the routine
%            brokenbasis.
%            If the size is [nutilde*n] x nutilde, then line
%            (k-1)*nutilde+q+1 contains the values of the q-th derivatives of
%            the nutilde smooth functions in knot k. (k=1,2,...,n)
%            If the size is [nutilde*(n-1)] x nutilde, then line
%            (k-1)*nutilde+q+1 contains the values of the q-th derivatives of
%            the nutilde smooth functions in knot k+1, assuming that the
%            derivatives in knot k were q (local definition)
%    OmegaL  real matrix of size [nutilde*(n-1)] x nutilde
%            containing the function values and derivatives at the left
%            end point of each intraknot interval (local definition)
% Description
%    The objective of this routine is to handle several possible specifications
%    of the smooth set of functions, so that other routines like brokenbasis
%    may work with one standard representation, matrices with sizes specified
%    above
% See also
%    help brokenbasis
%    help smoothsetmoments
%    help aggregatesmoothset

if nargin<2, Omega=1; end
switcharguments = false;
if iscell(knots), switcharguments = true; 
elseif min(size(knots))>1, switcharguments = true;
end
if switcharguments,
   dummy=Omega; Omega=knots;
   if nargin>1, knots=dummy; else, knots = NaN; end
end
n = length(knots);
if ~isnan(knots(1)),
   if n==1, n=knots; knots = (0:n-1)/(n-1); end
   if isnumeric(Omega), if max(size(Omega))==1, 
      % argument Omega actually contains nutilde
      nutilde=Omega;
      % construct local power functions (on each interknot segment)
      gaps = knots(2:n)-knots(1:n-1);
      nnutilde = (n-1)*nutilde;
      Omega = zeros((n-1)*nutilde,nutilde);
      Omegad = ones(n-1,nutilde);
      for q=1:nutilde-1, Omegad(1:n-1,q+1) = gaps.^q; end
      Omega(1:nutilde:nnutilde,1:nutilde) = Omegad;
      D = diag(0:nutilde-1);
      for d=1:nutilde-1,
         Omegad = [zeros(n-1,1) Omegad(1:n-1,1:nutilde-1)];
         Omegad = Omegad*D;
         Omega(d+1:nutilde:nnutilde,1:nutilde) = Omegad;
      end
   end, end
end
if isnumeric(Omega),
   if size(Omega,1)<size(Omega,2), Omega=Omega'; end
   nutilde = size(Omega,2);
   if ~ismember(size(Omega,1),nutilde*(n-1:n)) & ~isnan(knots),
      Omega = {Omega}; % make Omega ready for further processing
   end
end
% Now check if Omega carries any other than numeric information
agg = false; OmegaL = NaN; LR = false;
if iscell(Omega), nO = length(Omega); O = {}; k=1;
   while k<=nO, vark = Omega{k};
      if isnumeric(vark),
         O = {O{:},vark}; k=k+1;
      elseif ischar(vark), switch(vark)
         case {'agg','local'},
            agg = true; k = k+1;
         case {'LR'}, % Omega(1,2) should be interpreted as {OmegaL,Omega};
                      % no further action by this routine is required 
            LR = true; k = k+1;
         otherwise, % do nothing, argument is considered to be 'void' or 'none'
             k = k+1;
         end
      else, % do nothing, argument is considered to be 'void' or 'none'
         k = k+1;
      end
   end
   Omega = O;
end
if LR, OmegaL = Omega{1}; Omega = Omega{2}; nutilde = size(Omega,2); end
if iscell(Omega),
   % convert cell representation of Omega into matrix representation
   % nutilde = min(size(Omega{1}));
   nutilde = size(Omega{1},2);
   if isnan(knots), n = max(size(Omega{1})); knots=(0:n-1)/(n-1); end
   d = length(Omega)-1;
   if d<nutilde-1,
      % complete numerically (if necessary) the q-th derivatives of Omega in all
      % points x for q=d+1 to nutilde-1. 
      % Reported bug: This works fine on equidistant x, but fails on 
      %               nonequispaced x when more than one derivative is computed.
      domdx = Omega{d+1};
      if ismember(n,size(domdx)),
         dx = initmom(knots);
      else % Omega{d+1} is supposed to be defined on a fine equidistant grid
         n = size(domdx,1);
         dx = ones(n,1)/n; dx([1 n]) = dx([1 n])/2;
      end
      for q=d+1:nutilde-1,
         domdx(1:n,1:q) = 0;
         for k=q+1:nutilde, domdx(1:n,k) = initmom(domdx(1:n,k))./dx; end
         Omega = {Omega{:},domdx};
      end
   end
   nO = size(Omega{1},1);
   O = zeros(nutilde*nO,nutilde);
   for k=1:nO,
      for q = 1:nutilde,
         O((k-1)*nutilde+q,1:nutilde) = Omega{q}(k,1:nutilde);
      end
   end
   Omega = O;
end

if nargout==1,
   OmegaL=Omega;
elseif isnan(OmegaL(1,1));
   nnutilde = size(Omega,1);
   if nnutilde==(n-1)*2*nutilde,
      L = find(mod(floor((0:nnutilde-1)/nutilde),2)==0);
      R = find(mod(floor((0:nnutilde-1)/nutilde),2)==1);
      OmegaL = Omega(L,1:nutilde); Omega = Omega(R,1:nutilde);
      nnutilde = (n-1)*nutilde;
   elseif nnutilde==(n-1)*nutilde,
      OmegaL = repmat(diag(factorial(0:nutilde-1)),n-1,1);
   elseif nnutilde==n*nutilde,
      if agg==true,
         OmegaL = repmat(Omega(1:nutilde,1:nutilde),n-1,1);
         Omega = Omega(nutilde+1:nnutilde,1:nutilde);
         nnutilde = (n-1)*nutilde;
      elseif nargout==2,
         OmegaL = Omega(1:nnutilde-nutilde,1:nutilde);
         Omega = Omega(nutilde+1:nnutilde,1:nutilde);
      end
   end
end
