
function y = IRT_2Gspline(x,w,nt,n,varargin);

%  IRT_2Gspline -- 1D inv. nondecimated spline wav.transf. on noneq. points
%   Usage
%     y = IRT_2Gspline(x,w,nt,n,alfa,<other arguments>);
%   Inputs
%     x      grid-locations of observations (time, space)
%     w      wavelet coefficients
%     nt     dual vanishing moments --  order of B-spline scaling function
%     n      primal vanishing moments (0-moments of primal wavelet function)
%     alfa   scalar, vector, or cell (default 1/2) for avaraging cycle spins
%     <other arguments>: none have been implemented in current version
%   Outputs
%     y      vector of size(w) with reconstructed signal/function/observations
%   Description
%     if alfa is a vector, then the first elements of alfa are those used at
%     the coarsest scales. In other words, alfa is interpreted coarse to fine
%   Note
%   See also
%     help RWT_2Gspline (forward non-decimated transform)
%     help IWT2G1D, option 'Bspline' (decimated = fast CDF wavelet transform)

% Set default values

alfa = 1/2; alfacell = {};
% Parsing varargin
nvarargin = length(varargin);
if nvarargin>0,
   if isnumeric(varargin{1}), alfa = varargin{1}; 
   elseif iscell(varargin{1}), alfacell = varargin{1};
   end
end
vararginsentinel = {varargin{:},NaN};
for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'todo'}
     warning(['no implementation for ' vark])
   otherwise
     warning(['no implementation for ' vark])
end, end, end


transposex = false;
if size(x,2) > size(x,1),
   x = x'; transposex = true;
end
N = length(x);
% if w is a matrix, then turn it into a cell array
transposew = false;
if isnumeric(w),
   % w is a matrix or a vector
   % Three possible cases: 
   % (1) it is a matrix, where each column or each row corresponds to one
   % resolution level. In that case, one of the dimensions is M+1, where M is
   % the number of resolution levels. We assume that M+1<N
   % (2) It is a vector of length N*(M+1) containing all the coefficients
   % (3) It is a matrix, where one of the dimensions is N*(M+1). Let K be the
   % other dimension. Then the input is the RWT of a N x K or K x N matrix
   wmatrix = w;
   [sizew1 sizew2] = size(wmatrix);
   minsizew = min(sizew1,sizew2); maxsizew = max(sizew1,sizew2);
   if minsizew < N & minsizew > 1,
      if maxsizew ~= N,
         error('incompatible sizes w and x')
      elseif sizew1 ~= N,
         wmatrix = wmatrix'; sizew2 = sizew1; transposew = true;
      end
      M = sizew2-1; K = 1; w = cell(1,M+1);
      for j=1:M+1, w{j} = wmatrix(1:N,j); end
   else
      if maxsizew>N*floor(maxsizew/N),
         error('incompatible sizes w and x')
      elseif sizew1<maxsizew, wmatrix = wmatrix'; transposew = true;
      end
      K = minsizew; M = maxsizew/N-1; k = 0; w = cell(1,M+1);
      for j=1:M+1, w{j} = wmatrix(k+1:k+N,1:K); k = k+N; end
   end
else
   M = length(w)-1;
end

% set vector of alfa
if length(alfacell)==0, alfacell = num2cell(alfa); end
nac = length(alfacell);
% alfacell is interpreted "coarse scales first", except if its first
% element is a vector of length larger than one. In that case, it is
% interpreted "fine scales first". Since alfacell is processed fine to coarse,
% its order is inverted in all other cases.
if length(alfacell{1})==1, alfacell = alfacell(nac:-1:1); end
alfavec = row(alfacell{1});
for c=2:nac, alfavec = [alfavec row(alfacell{c})]; end
nav = length(alfavec);
alfavec = [alfavec alfavec(nac)*ones(1,2^M-1-nav)];


o = sparse(zeros(N,1));
infofromRWT = RWT_2Gspline(x,o,nt,n,M,varargin{:},'addoutputinfo');
finalupdates = infofromRWT{1};
% future versions might work with:
% liftedrefinements = infofromRWT{2};

% reconstruct; i.e., carry out the inverse transform
y = w{M+1};
C = size(y,2);
for j = M:-1:1,
   finalupdatesj = finalupdates{j};
   skip = 2^(j-1);
   wj = w{j};
   for k=1:skip,
      alfa = alfavec(end); alfavec = alfavec(1:end-1);
      xk = x(k:skip:N);
      Njk = length(xk);
      yk = cell(2);
      [e o] = splitevenodd(xk);
      for eo=0:1,
         ye = y(k+skip*eo:2*skip:N,1:C);
         yo = wj(k+skip*(1-eo):2*skip:N,1:C);   
         U = finalupdatesj{k+skip*eo};
         if max(max(abs(U)))>0, ye = ye - U*yo; end
         ykeo = zeros(Njk,C);
         ykeo(o,1:C) = yo; ykeo(e,1:C) = ye; 
         UP = nondecimatedliftingsteps('liftingstepsBspline',xk,nt,'shift',eo);
         ykeo = invliftingsteps(ykeo,UP,e); yk{eo+1} = ykeo;
         % PREPARE SECOND PART OF NONDECIMATED TRANSFORM (cycle spin)
         % switch roles even-odd
         ecopy = e; e = o; o = ecopy;
      end % for eo=1:2
      y(k:skip:N,1:C) = alfa*yk{1} + (1-alfa)*yk{2};
   end      
end

if transposew, y = y'; end
