
function UP = liftingstepsBspline(x,ptilde)

% liftingstepsBspline - ThreshLab2/Irregular/1D/ --- 
%          finds lifting matrices in dyadic two-scale equation for B-splines
%          (non-robust version against coinciding knots)
% Usage
%   UP = liftingstepsBspline(x,ptilde);
% Input
%   x       vector of knots
%   ptilde  order of splines; ptilde=1: constant (Haar-scaling functions)
%                             ptilde=2: linear splines
%                             ptilde=4: cubic splines
% Output
%   UP    cell array of matrices
% Description
%   The elements of UP constitute a matrix H
%   found by: H = refinementfromliftingsteps(UP);
%   so that: PhiJ*H = PhiL is the refinement (or two-scale) equation
%   In this equation PhiJ is spline basis, order p, on knots x
%                    PhiL is spline basis, order p, on knots x(even)
%                    where even = splitevenodd(x)
% Note
%   The implementation slightly deviates from the paper on Nonequispaced
%   B-spline wavelets in the Theorem (wavelet transform from lifting
%   factoring). This implementation adopts the defintion P = H_{j,o} for
%   the final prediction step, while the paper uses P = H_{j,o}/D.
% See also
%   help refinementfromliftingsteps

n = length(x);
x = column(x);
nU = floor((ptilde+1)/4); 
r = ceil(ptilde/2) - 2*nU;
rtilde = mod(ptilde,2);

% Add coincident knots in front and at the rear. These points are used for
% calculations of liftingsteps in existing knots, but no lifting is taking
% place in the newly added knots
x1 = repmat(x(1),ptilde,1);
xn = repmat(x(n),ptilde+4,1);
if ptilde==1, xn = repmat(2*x(n)-x(n-1),ptilde+4,1); end
% ptilde==1, i.e., the Haar transform; if length(x) is even, the last point is
% estimated by extrapolation. We have to add a point different from x(end) in
% order to avoid NaN = [y(end)-y(end)]/[x(end)-x(end)] in the prediction
xx = [x1; x; xn]; 
[e o] = splitevenodd(x); 
ne = length(e); no = length(o); ee = ptilde+e; oo = ptilde+o;
% the first ptilde knots have been added artificially

nUP = 2*nU+r;
UP = cell(1,nUP+1);
cc = ones(size(xx));
s = 0;
if r==1, % i.e., if nUP is odd, then start with a prediction step
   P = sparse(zeros(no,ne));

   % s=0; m=nU; t=r-1=0;
   W  = xx(ee+2*nU+2)-xx(ee-2*nU+rtilde*min(1+s,2));
   P0 = (xx(ee+2*nU+2)-xx(ee+2*nU+1))./W;
   P1 = (xx(ee-2*nU+1)-xx(ee-2*nU+rtilde*min(1+s,2)))./W;

   P0 = P0(1:no); P1 = P1(1:ne-1);
   P(1:no,1:no) = diag(P0);
   P(1:ne-1,2:ne) = P(1:ne-1,2:ne) + sparse(diag(P1));

   UP{1} = P;
   
   if nU>0,
      cc(ee+1) = (xx(ee+2*nU+1)-xx(ee-2*nU+1))./W; 
   end
end

for s = 1:nU, 
   m = nU-s;
   t = r+2*s-1;

   % (1) UPDATE
   U = sparse(zeros(ne,no));
   W = xx(ee+2*m+2*t)-xx(ee-2*m-2*t+rtilde*min(r+s,2));
   V0 = -(xx(ee-2*m-1)-xx(ee-2*m-2*t+rtilde*min(r+s,2)))./W;
   V1 = -(xx(ee+2*m+2*t)-xx(ee+2*m+1))./W;
% next 2 lines to be tested/ cc(ee+1) <-> cc(ee-1) ???
   U0 = V0.*cc(ee)./cc(ee+1);
   U1 = V1.*cc(ee)./cc(ee-1);

   U0 = U0(1:no); U1 = U1(2:ne);
   U(1:no,1:no) = diag(U0);
   U(2:ne,1:ne-1) = U(2:ne,1:ne-1) + sparse(diag(U1));

   cc(ee) = cc(ee).*(xx(ee+2*m+1)-xx(ee-2*m-1))./W;
   UP{2*s-1+r} = U;

   % (2) PREDICTION
   P = sparse(zeros(no,ne));
   W = xx(ee+2*m+2*t+2)-xx(ee-2*m-2*t+rtilde*min(1+s,2));
   V0 = (xx(ee+2*m+2*t+2)-xx(ee+2*m+1))./W;
   V1 = (xx(ee-2*m+1)-xx(ee-2*m-2*t+rtilde*min(1+s,2)))./W;
   if s<nU, % Due to the alternative definition of the final prediction step P
            % in the factoring, compared to the paper on Nonequispaced B-spline
            % wavelets, we have to fill in P with different expressions when
            % s==nU (=last step)
      P0 = V0.*cc(ee+1)./cc(ee);
      P1 = V1.*cc(ee+1)./cc(ee+2);
   else % cc(ee) is kept for use in D (rescaling)
      P0 = V0.*cc(ee+1); 
      P1 = V1.*cc(ee+1);
   end

   P0 = P0(1:no); P1 = P1(1:ne-1);
   P(1:no,1:no) = diag(P0);
   P(1:ne-1,2:ne) = P(1:ne-1,2:ne) + sparse(diag(P1));

   if s<nU,
      cc(ee+1) = cc(ee+1).*(xx(ee+2*m+1)-xx(ee-2*m+1))./W;
   end
   UP{2*s+r} = P;
end

D = sparse(diag(cc(ee)));
UP{nUP+1} = D;
