function [z, Ne] = IMLPT2D(x,y,w,M,varargin);

% IMLPT2D -- ThreshLab2/Irregular/MultiscaleKernel -- ThreshLab's routine for
%            the Inverse Multiscale Local Polynomial Transforms, 2D, which is
%            a Burt-Adelson's pyramid on irregular point sets with
%                prefilter H, prediction P, update U
%  Usage
%    z = IMLPT2D(x,y,w,M,varargin);
%  Inputs
%    x      x-coordinate of observations
%    y      y-coordinate of observations
%    w      wavelet/pyramid coefficients
%    M      number of resulution levels
%    varargin several options (see below)
%  Outputs
%    z      vector of size(x) with reconstructed observations/signal z(x,y)
%    Ne     (optional) number of scaling coefficients (dependent on M)
%  Description
%    If w is a matrix, then the transform is performed on its COLUMNS
%  NOTE
%    The prefilter H has not yet been implemented
%  See also
%    help FMLPT2D (forward transform)
%    help IMLPT2D1step (iterated 1 scale transform)
%    help IMLPT1D (1D version)

x = column(x); y = column(y);
N = length(x);
if size(y,1) ~= N, error('dimension mismatch x and y'), end
ztranspose = 0;
if size(w,1) == 1, w = w'; ztranspose = 1; end
ncols = size(w,2);

n = [0 0]; % default value; may be overwritten by subsequent statement
moments = [];
Ne = NaN;
nvarargin = length(varargin);
splitrate = 2;
bandwidths = NaN;
for k = 1:nvarargin-1, vark = varargin{k}; if ischar(vark), switch(vark)
    case {'n','nU','primom','primalmoments','nprimal','nprimalmoments'}
       n = varargin{k+1};
    case {'moments','Moments'}
       moments = varargin{k+1};
    case {'Ne'}
       Ne = varargin{k+1};
    case {'splitrate'}
      splitrate = varargin{k+1};
    case {'bandwidth','bandwidths','h','hj','hJ'}
      bandwidths = varargin{k+1};
end, end, end

% Before doing the inverse transform, we first need to recompute parameters
% along the forward transform: (1) the moments (2) the grids (3) the bandwiths
% at each scale

o = sparse(zeros(size(x)));
infofromforwardtransform = ...
    FMLPT2D(x,y,o,M,varargin{:},'U',0,'Ht',1,'addoutputinfo');
moments = infofromforwardtransform{1};
grids = infofromforwardtransform{2};
bandwidthcells = infofromforwardtransform{3};
nbandwidthsperlevel = length(bandwidthcells{1});
bandwidths = zeros(nbandwidthsperlevel,M);
for j = 1:M, 
   bandwidths(1:nbandwidthsperlevel,j) = column(bandwidthcells{j});
end

nm = size(moments,2);

% Now, do the inverse transform
onestepvarargin = varargin;
% every step in the reconstruction overwrites the details and throws away the
% evens (see flow diagrams for lifting scheme)
coarse = grids{1};
Ne = length(coarse);
xcoarse = x(coarse);
ycoarse = y(coarse);
if any(n) > 0,
   Mcoarse = moments(1:Ne,1:nm);
else
   Mcoarse = [];
end
sj = w(1:Ne,1:ncols);
k0o = Ne;
k0e = 0;

for j = 1:M,
   fine = grids{j+1};
   even = grids{j};
   even = find(ismember(fine,even));
   No = length(fine);
   xfine = x(fine); yfine = y(fine);
   if n>0,
      Mfine = moments(k0o+1:k0o+No,1:nm);
   else
      Mfine = [];
   end
   wj = w(k0o+1:k0o+No,1:ncols);

   [sj varargout] = IMLPT2D1step(xfine,yfine,even,wj,sj,Mfine,Mcoarse,...
                       onestepvarargin{:},...
                       'bandwidth',bandwidths(:,j));
   % prepare for next step
   k0e = k0o; k0o = k0o+No; Ne = No;
   coarse = fine; xcoarse = xfine; ycoarse = yfine; Mcoarse = Mfine;
   onestepvarargin = varargout;
end

z = sj;
if ztranspose, z = z'; end

