function [sfine outputoptions] = ...
     IMLPT2D1step(xfine,yfine,even,wcoarse,scoarse,Mfine,Mcoarse,varargin);

% IMLPT2D1step -- ThreshLab2/Irregular/MultiscaleKernel -- One step in
%                ThreshLab's routine for scattered data reconstruction from a
%                2D Multiscale Local Polynomial Decomposition (FMLPT2D)
%  Usage
%     [sfine outputoptions] = ...
%     IMLPT2D1step(xfine,yfine,even,wj,sj,Mfine,Mcoarse,varargin);
%  Inputs
%    xfine    x-coordinates of fine scale grid
%    yfine    y-coordinates of fine scale grid
%    even     index vector into xfine for coarse scale grid-locations
%    wcoarse  detail/wavelet coefficients or offsets at coarse scale
%    scoarse  scaling coefficients at coarse scale
%    Mfine    fine scale scaling function moments
%    Mcoarse  coarse scale scaling function moments
%  Outputs
%    sfine    fine scale scaling coefficients
%  Description
%    Applies one step inverse lifting filter bank with an optional prefilter,
%    at most one prediction step and at most one update step. 
%  See also
%    help IMLPT2D

if size(xfine,2) > size(xfine,1),
   xfine = xfine';
end
if size(yfine,2) > size(yfine,1),
   yfine = yfine';
end
dotranspose = 0;
if size(wcoarse,1) == size(scoarse,1),
   wcoarse = wcoarse';
   scoarse = scoarse';
   dotranspose = 1;
end

x = xfine; xeven = x(even);
y = yfine; yeven = y(even);
N = length(x);
if length(y) ~= N, error('sizes xfine and yfine dismatch'), end
n = size(Mfine,2);
% following values are the default values; they may be overwritten while
% parsing through varargin
H = 0;
P = 1;
U = 0; if n>0, U = 'primal'; end
nt = [1 1];
bandwidthsx = (max(x) - min(x)) * 3/sqrt(N);
bandwidthsy = (max(y) - min(y)) * 3/sqrt(N);
bandwidths = [bandwidthsx; bandwidthsy];
% if all x (or all y) are equal, then take bandwidth 1:
constantcoordinate = abs(bandwidths)<eps;
bandwidths(find(constantcoordinate)) = 1;
kerneltype = 'cos';
prefilteroptions = {};
adaptive = 0;
reorder = 0;
splitrate = 2;
splitscheme = 'splitbypeel';
bandwidthfactor = NaN;


% Parsing varargin
nvarargin = length(varargin);
outputoptions = {};
otheroptions = {};
k = 1;
while k <= nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'split','splitscheme','partitioning','takeoff','takeaway'}
      splitscheme = varargin{k+1};
      k = k+2;
   case {'splitrate'}
      splitrate = varargin{k+1};
      k = k+2;
   case {'nt','ntilde','ntP','dualmom','dualmoments','ndual','ndualmoments'}
      nt = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'n','nU','primom','primalmoments','nprimal','nprimalmoments'}
      % n is passed implicitly by the size of Mfine, no need to pass on n
      % explicitly; ignore this declaration
      warning('explicit number of primal moments ignored')
      k = k+2;
   case {'H','Hfilter','prefilter','prefilt'}
      Ht = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'P','Pmethod','prediction','Prediction','Predict','predict'}
      P = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'U','update'}
      U = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'bandwidth','bandwidths','h','hj','hJ'}
      bandwidths = varargin{k+1};
      % bandwidth is not returned in the output; this implementation is
      % different from the approach in IMLPT1D1step.m 
      % The 2D inverse transform deals with bandwidths at the level of
      % IMLPT2D.m, whereas the 1D version handles bandwidths at the level of
      % the one-step transforms (the 1D version is older)
      k = k+2;
   case {'kerneltype','type'}
      kerneltype = varargin{k+1};
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   case {'prefilteroptions','prefiltoptions','Htopt','prefiltoption',...
         'prefiltopt'}
      Hoption = varargin{k+1};
      if iscell(Hoption),
         prefilteroptions = {prefilteroptions{:},Hoption{:}};
      else
         prefilteroptions = {prefilteroptions{:},Hoption};
      end
      outputoptions = {outputoptions{:},varargin{k:k+1}};
      k = k+2;
   otherwise % the argument cannot be interpreted by this routine and it is
             % passed to the next level
      outputoptions = {outputoptions{:},vark};
      otheroptions = {otheroptions{:},vark};
      k = k+1;
   end
   else % if ischar(vark)
      outputoptions = {outputoptions{:},vark};
      otheroptions = {otheroptions{:},vark};
      k = k+1;
end, end

% Design of the transform
  % PREDICTION MATRIX %
  Ne = length(xeven);
  Ie = sparse(eye(Ne));
  if ischar(P), Pmethod = P; P = [P '     '];
     switch(P(1:6))
     case {'locpol','localp'}
        Pvars = {x,y,xeven,yeven,Ie,bandwidths,kerneltype,nt-1};
        Pmethod = 'locpolestimation2D';
     otherwise
        Pvars = {'x',x,'y',y,'X',xeven,'Y',yeven,'Z',Ie,...
                 'bandwidths',bandwidths,'degree',nt-1};
     end
     if exist(Pmethod,'file') ~= 0,
        Pvars = {Pvars{:},otheroptions{:}};
        P = sparse(feval(Pmethod,Pvars{:})');
     else
        P = 1;
     end
  end
  if isscalar(P), P = sparse(diag(P*ones(1,N))); P = P(1:N,even); end
  % PREFILTER %
  % not yet implemented
  H = 0;
  % UPDATE %
  if ischar(U), U = [U '     '];
  switch U(2:6),
  case {'oment','rimmo','rimal'} % minimum 
     if n>0,
        n = min(n,N);
        Mf = Mfine(1:N,1:n); Mc = Mcoarse(1:Ne,1:n);
        Io = sparse(eye(N));
        % At this moment no reordering has been implemented (cfr.
        % FMLPT1D1step.m). Such a reordering would take the form of a
        % computation of a center of mass for all basis functions, in order to
        % identify or redefine the closest neighbours
        xo = x; yo = y; xe = xeven; ye = yeven;
        U = updateprimalmoments2D(xo,yo,Io,xe,ye,Mf,Mc);
     else
        U = 0;
     end
  case {'innor'} % minimum norm solution within 
     disp('not yet implemented')
  otherwise
     U = 0;
  end, end
  if isscalar(U), U = sparse(diag(U*ones(1,N))); U = U(even,1:N); end

% Execution of transform
  % PREFILT %
  Hwcoarse = H*wcoarse;
  % SUBSAMPLE AND SUBTRACT%
  scoarse = scoarse-Hwcoarse(even,:);
  % UPDATE %
  scoarse = scoarse - U*wcoarse;
  % PREDICT %
  wcoarse = wcoarse + P*scoarse;

% Filling in
% Prepare next step
  sfine = wcoarse;
