
function [yk hh] = extlocpolestimation00(varargin);

% extlocpolestimation00 - ThreshLab2/Irregular/1D/ --- Extended and extendible
%                         local polynomial smoothing routine (see notes below)
%   Usage
%      yk = extlocpolestimation00('x',x,'X',X,'Y',Y,others)
%      [yk hh] = extlocpolestimation00('x',x,'X',X,'Y',Y,others)
%      
%   Inputs
%      'x',x                x-coordinates in which kernel is evaluated 
%                           (x is real vector)
%      'x0',x0 or 'X',X     x-coordinates in smoothing curve's definition
%                           (x0/X is real vector)
%      'y0',y0 or 'Y',Y     y-coordinates in smoothing curve's definition
%                           (y0/Y is real vector; length(x0)=length(y0))
%      'degree',n           polynomial degree
%                           (n is positive integer or zero)
%      'bandwidth',h        default bandwidth (can be modified locally; see
%                           below) 
%                           (h is real number or a vector of length(x))
%                           (can also be declared 'bandwidths','h','hj','hJ')
%      'kerneltype',type    type of kernel; type is string:
%      or: 'type',type                      cfr. list in `help evalkernel'
%      'side',s             with s in {-1,0,1}: left, both sides, right
%                           (default is 0)
%      'convex',c           with c in {0,1} (default 0)
%   Outputs
%      yk     local polynomial smoothing evaluated in x
%      hh     locally modified bandwidth
%   Notes
%      (1)
%      x, x0 and y0 are supposed to be arranged as rows.
%      if y0 is a square matrix of size equal to the length of x0, then the
%      smoothing operation is performed on all ROWS.
%      (2)
%      This routine extends locpolestimation00, and allows easy further
%      extensions. The original motivation for the new implementation has been
%      the need for local flexibility in the bandwidth (in contrast to a fixed
%      bandwidth within each scale) which allows to impose convex local
%      polynomial coefficients (not provided by locpolestimation00.m)
%      Convex prediction coefficients are important, not only for the
%      stability of the prediction itself, but also because the prediction may 
%      play a role in the definition of the prefilter.
%      Therefore, locpol may generate much worse coefficients than extlocpol
%   See also
%      help evalkernel
%      help locpolestimation00
%      help kernelestimation


% default values
side = 0;
kerneltype = 'cos';
convex = false;
flexbandwidth = false;
degree = 0;

% Parsing varargin
nvarargin = length(varargin);
listofoptions = {};
vararginsentinel = {varargin{:},'sentinel'};
for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'x'}
      x = vararginsentinel{k+1};
   case {'x0','X'}
      x0 = vararginsentinel{k+1};
   case {'y0','Y'}
      y0 = vararginsentinel{k+1};
   case {'degree'}
      degree = vararginsentinel{k+1};
   case {'bandwidth','bandwidths','h','hj','hJ'}
      h = vararginsentinel{k+1};
   case {'kerneltype','type'}
      kerneltype = vararginsentinel{k+1};
   case {'side'}
      side = vararginsentinel{k+1};
   case {'flexbandwidth','flexiblebandwidth'},
      flexbandwidth = true;    
   case {'convex'}
      convex = true; B = vararginsentinel{k+1};
      if islogical(B)||isnumeric(B),
         convex = B;
      end
end, end, end

if exist('y0') ~= 1, % for density estimation
   y0 = ones(size(x0))/length(x0); 
end 

x = row(x);
x0 = row(x0);

nobs = length(x0);
neval = length(x);
[nrows ny] = size(y0);
if ny ~= nobs,
   if nrows ~= nobs, error('dimension mismatch x0 and y0')
   else nrows = ny; y0 = y0'; end
end

if ischar(side),
   switch side(1)
   case {'R','r'},
      side = 1;
   case {'L','l'},
      side = -1;
   otherwise,
      side = 0;
   end
end

if convex == true,
   flexbandwidth = true;
end

hh = h;
if length(h) == 1, hh = h*ones(size(x)); end
switch kerneltype
case {'Gaussian','gaussian','gauss','Gauss'}
   kernelsupport = Inf*ones(size(hh));
otherwise
   kernelsupport = hh;
end

yk = zeros(nrows,neval);
sk = zeros(size(x));
for i = 1:neval,
   hi = hh(i);
   io = (abs(x0-x(i))<kernelsupport(i));
   % iL is set of points in x0 left of x(i)
   iL = (x0<x(i)+eps);
   iR = (x0>x(i)-eps);
   % iM = (abs(x0-x(i))<eps);
   ioR = find(io & iR);
   ioL = find(io & iL);
   iR = find(iR);
   iL = find(iL);
   io = find(io);
   if (isempty(ioR) && flexbandwidth && ~isempty(iR)),
      [xR Rmin] = min(x0(iR)); Rmin = iR(Rmin); ioR = Rmin; io = [io ioR];
      hi = max(hi,xR-x(i));
   end
   if isempty(ioL) && flexbandwidth && ~isempty(iL),
      [xL Lmax] = max(x0(iL)); Lmax = iL(Lmax); ioL = Lmax; io = [ioL io];
      hi = max(hi,x(i)-xL);
   end
   % We again increase the local bandwidth if there is only one point on one
   % side, to avoid that this point has a too low weight
   if flexbandwidth,
      B = 1.1;
      if length(ioR) == 1, hi = max(hi,B*(x0(ioR)-x(i))); end
      if length(ioL) == 1, hi = max(hi,B*(x(i)-x0(ioL))); end
   end
   if side == 1, io = ioR;
   elseif side == -1, io = ioL;
   end
   hh(i) = hi;
   if ~isempty(io),
      xo = x0(io); 
      wo = evalkernel(x(i)-xo,hi,kerneltype);
      p = min(degree,length(xo)-1);
      Io = eye(length(xo));
      if convex == true,
         co = poscoefweightedLSpol(x(i),xo,Io,wo,p);
      else
         co = weightedLSpol(x(i),xo,Io,wo,p);
      end
      jo = find(max(abs(y0(1:nrows,io)),[],2)>eps);
      yo = y0(jo,io);
      yo = column(yo);
      yk(jo,i) = co*yo;
   end
end
