
function H = findbiorthfilter(P,M,midpoint);

% findbiorthfilter -- finds biorthogonal filter, given vanishing moments 
%  Usage
%    H = findbiorthfilter(P,M,midpoint);
%    H = findbiorthfilter(P,M);
%    H = findbiorthfilter(P);
%    H = findbiorthfilter(P,[],midpoint);
%  Inputs
%    P          n x m matrix, with n>m or n=m
%    M          p x n matrix
%    midpoint   central point in each column of P
%  Outputs
%    H          m x n matrix
%  Description
%    finds a sparse solution to the system H*P = I, under the side condition
%    that M'*(I-P*H) = 0. 
%    This proceeds in 3 steps:
%    1) find a sparse solution H1 for H*P = I, using sparsesolveAXisB
%    2) find a sparse homogeneous solution H0 for H*P = 0
%    3) find a sparse solution H = H1+X*H0 satisfying M'*(I-P*H) = 0
%       This is a system in X, which is M'*(I-P*H1) = M'*P*X*H0,
%       which is of the general form A*X*H0 = B,
%       where B = M'*(I-P*H1) and A = M'*P
%       The solution of this system proceeds in two steps
%       3a. U*H0 = B, or H0'*U' = B', which is an overdetermined system
%       3b. A*X = U, which is underdetermined, so imposing sparsity on X is
%                    posible here
%  Note
%  See also
%    help sparsesolveAXisB
%    help findsparsenullspace
%    help findmidpointsinsparsematrix
%    help prefiltbyvanmom (for a relaxed version)

if nargin < 3,
   midpoint = findmidpointsinsparsematrix(P');
end

if nargin < 2,
   M = [];
end

[n m] = size(P);
Im = eye(m);
H1 = sparsesolveAXisB(P',Im,midpoint,(1:m))';

if isempty(M) | isnan(M),
   H = H1;
else
   midpointrow = midpoint;
   midpointcol = zeros(1,n);
   midpointcol(midpointrow) = 1;
   midpointcol = cumsum(midpointcol);
   odd = (2:2:n);
   H0 = findsparsenullspace(P',midpointrow,midpointcol,odd)';

   In = eye(n);
   B = M'*(In-P*H1);
   U = ((H0*H0')\(H0*B'))';
   A = M'*P;
   X = sparsesolveAXisB(A,U,0,(1:n));
   H = H1+X*H0;
end

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
