
function [fhat knots H] = finescalehist(x,X,hmin,x0,x1,knots_in_the_middle);

% finescalehist - ThreshLab2/MultiscaleKernel - fine scale adaptive bin
%                                               histogram
%   Usage
%      fhat = finescalehist(x,X,hmin,x0,x1);
%      fhat = finescalehist(x,X,hmin,[x0 x1]);
%      fhat = finescalehist(x,X,hmin);
%      fhat = finescalehist(x,X)
%      fhat = finescalehist(x,X,{hmin,smax},...);
%      [fhat knots] = finescalehist(...)
%      fhat = finescalehist(x,XF,...)
%   Inputs
%     x        points in which histogram is evaluated
%     X        observations
%     XF       cell containing vector of x and corresponding F_X(x)
%     hmin     minimum bin width (default 0 may lead to instabilities)
%     smax     integer; maximum number of binned values
%     x0       minimum value of X (default min(X))
%     x1       maximum value of X (default max(X))
%   Outputs
%     fhat     vector/matrix of size(x)
%     knots    (optional) bin boundaries
%     H        filter so that 1./fhat = H * (1./X)
%   Description
%   Note
%   See also
%     help kernelestimation for options with adaptive bandwidths and k-nearest
%                           neighbour estimation

findfilter = (nargout>2); % for large datasets, the matrix H may be too large
                          % for storage; so we only keep track of this matrix
                          % if asked for
if nargin < 6, knots_in_the_middle = true; end
if nargin < 3, hmin = 0; end
F = NaN;
smax = Inf;
if iscell(X), F = X{2}; X = X{1}; end
if iscell(hmin), smax = hmin{2}; hmin = hmin{1}; end
X = row(sort(X));
F = row(F);
n = length(X);
if isnan(F), F = ((0:n-1)+0.5)/n; end
if nargin<4, x0 = X(1); end
if nargin<5, 
   if length(x0) == 1, x1 = X(n); 
   else, x1 = x0(2); x0 = x0(1);
   end
end
if knots_in_the_middle,
   knots = [x0 (X(2:n)+X(1:n-1))/2 x1]; nknots = n+1;
   Fknots = [0 (F(2:n)+F(1:n-1))/2 1]; 
else
   knots = [X x1]; nknots = n+1;
   Fknots = [F 1];
end
Deltax = knots(2:nknots)-knots(1:nknots-1);
DeltaF = Fknots(2:nknots)-Fknots(1:nknots-1);

Y = (n+1)*Deltax; h = (n+1)*hmin;
% Y ~ exp(f_X) (asymptotically)
% nobs = ones(1,nD); % number of observations between each pair of knots
N = DeltaF*n;

Nhr = N; XNhr = Y;
toonarrow = find(XNhr<h);
if findfilter, H = eye(nknots-1); end
for t = 1:length(toonarrow),
   k = toonarrow(t);
   s = 0;   
   while XNhr(k) < h && s < smax,
      s = s+1;
      l = k+ceil(s/2)*(-1)^s;
      if l<nknots && l > 0,
         XNhr(k) = XNhr(k)+Y(l); Nhr(k) = Nhr(k)+N(l);
         if findfilter, H(k,l) = 1; end
      end
   end
end
if findfilter, H = diag(1./Nhr)*H; end

fhat = Nhr./XNhr;
% normalise to have integral 1
fhat = fhat/(fhat*Deltax');
