
% This routine finds a few bases that come out from the multiscale
% kernel/local polynomial transforms
% The basis functions are the columns of the matrices defined in the code:
% PsiIntpolP4U0: cubic interpolating prediction, no update 
% PsiIntpolP4U2: cubic interpolating prediction, 2-taps update
% PsiKernelU0: kernel smoothing prediction, no update
% PsiKernelU2: kernel smoothing prediction, 2-taps update
% PsiLoclinU0: local linear smoothing prediction, no update
% PsiLoclinU2: local linear smoothing prediction, 2-taps update
% PsiLoclinU2convex: local linear smoothing prediction with convex pred.
% In order to plot a basis function, choose k and set
% plot(x,PsiIntpolP4U0(:,k)) and so on


warning off
if exist('gridnumber') ~= 1, gridnumber = 1; end
if exist('realrandom') ~= 1, realrandom = 0; end
if exist('samplesize') ~= 1, samplesize = 2067; end
if exist('onebandwidth') ~= 1, onebandwidth = true; end
n = samplesize;
x = makegrid(n,gridnumber,realrandom);

L = 0;
J = floor(log2(n)); M = J-L;

% next are parameters of the forward local polynomial lifting scheme
if exist('predkernel') ~= 1, predkernel = 'cos'; end
if exist('finescalebandwidth') ~= 1, 
   finescalebandwidth = (max(x) - min(x)) * 3/n;
end
if exist('bandwidths') ~= 1,
   if exist('h0') ~= 1, h0 = 1.05; end
   njj = n; nj = n; for j=2:M, nj = ceil(nj/2); njj = [njj nj]; end
   bandwidths = h0*log(njj)./njj; % heuristical choice
else
   onebandwidth = false;
end
if onebandwidth==true,
   hh = finescalebandwidth;
else
   hh = bandwidths;
end
% With pt = the number of dual vanishingmoments, the parameters 
% {predkernel,hh,pt} define the subdivision, they are the equivalent of the
% single parameter pt in a subdivision scheme with polynomial prediction
cubic = 4;
linear = 2;
constant = 1;

if exist('primmom') ~= 1, primmom = 2; end

textoptions = {'fontsize',14,'fontweight','b'};
plotoptions = {'linewidth',3};
bullets = {'o','markersize',6,'linewidth',2};


% par0 = {M,'P','locpol','kerneltype',predkernel,'bandwidth',hh,'pt',pt,'p',p};

I = eye(n);

[Wt, Ne] = FMLPT1D(x,I,M,'P','intpol','pt',constant);

nw = size(Wt,1);

I = eye(nw);

%%%%% Cubic interpolating polynomial prediction %%%%%
intpolP4U0 = {M,'P','intpol','pt',cubic};
PsiIntpolP4U0 = IMLPT1D(x,I,intpolP4U0{:});
intpolP4U2 = {intpolP4U0{:},'p',primmom};
PsiIntpolP4U2 = IMLPT1D(x,I,intpolP4U2{:});

%%%%% Kernel smoothing prediction %%%%%
kernelU0 = {M,'P','kernel','kerneltype',predkernel,'bandwidth',hh};
PsiKernelU0 = IMLPT1D(x,I,kernelU0{:});
kernelU2 = {kernelU0{:},'p',primmom};
PsiKernelU2 = IMLPT1D(x,I,kernelU2{:});

%%%%% Local linear prediction %%%%%
loclinU0 = {M,'P','locpol','kerneltype',predkernel,'bandwidth',hh,'pt',linear};
PsiLoclinU0 = IMLPT1D(x,I,loclinU0{:});
loclinU2 = {loclinU0{:},'p',primmom};
PsiLoclinU2 = IMLPT1D(x,I,loclinU2{:});
loclinU2bis = loclinU2;
loclinU2bis{3} = 'extlocpolestimation00';
PsiLoclinU2bis = IMLPT1D(x,I,loclinU2bis{:});
loclinU2convex = {loclinU2bis{:},'convex',1};
PsiLoclinU2convex = IMLPT1D(x,I,loclinU2convex{:});

% example of visualisation: the basis functions at scale j=3
Psi = PsiLoclinU2convex;
figure(1)
plot(x,Psi(1:n,9:17),plotoptions{:})
axy = axis; axy(2) = 1.1; axy(3) = max(axy(3),-axy(4)); axis(axy)
axis off
drawaxes
title('Basis functions',textoptions{:})

% check the integrals and moments
figure(2)
% xx = [x(1) (x(2:n)+x(1:n-1))/2 x(n)]; dx = xx(2:n+1)-xx(1:n);
dx = initmom(x);
plot(dx*PsiLoclinU2convex,bullets{:})
title('Integrals')

