
function zk = locpolestimation2D(x,y,x0,y0,z0,h,kerneltype,degree);

% locpolestimation2D - ThreshLab2/MultiscaleKernel --- 2D local polynomial
%                                                      smoothing
%   Usage
%      for local polynomial smoothing
%        zk = locpolestimation2D(x,y,x0,y0,z0,h,kerneltype,degree);
%      for local polynomial density estimation
%        fk = locpolestimation2D(x,y,X,Y,h,kerneltype,degree);   
%   Inputs
%      x      x-coordinates in which kernel is evaluated
%      y      y-coordinates in which kernel is evaluated
%      x0     x-coordinates of sample locations in smoothing problem
%      y0     y-coordinates of sample locations in smoothing problem
%      z0     sample observations in smoothing problem
%      (X,Y)  observations/sample vector in density estimation problem
%      h      kernel bandwidths (either a scalar or a pair)
%      kerneltype  string-type (pair of) variables, see evalkernel
%      degree [p q] degree of polynomial x^p*y^q (default [0 0])
%   Outputs
%      zk     local polynomial smoothing evaluated in (x,y)
%   Notes
%      x,y;x0,y0 and z0 are supposed to be arranged as rows.
%      if z0 is a square matrix of size equal to the length of x0, then the
%      smoothing operation is performed on all ROWS.
%   See also
%      help evalkernel
%      help locpolestimation00 (1D version)

nobs = length(x0);
neval = length(x);
if length(y) ~= neval, error('dimension mismatch x and y'), end
if length(y0) ~= nobs, error('dimension mismatch x0 and y0'), end

[nrows nz] = size(z0);
if nrows <= 2 & zy <= 2, % number of observations must be at least 2
   degree = kerneltype;
   kerneltype = h;
   h = z0;
   z0 = ones(1,nobs)/nobs;
   densityest = 1;
else
   densityest = 0;
   if nz ~= nobs,
      if nrows ~= nobs, error('dimension mismatch (x0,y0) and z0')
      else nrows = nz; z0 = z0'; end
   end
end

if length(kerneltype)==2,
   kerneltypex = kerneltype{1};
   kerneltypey = kerneltype{2};
else
   kerneltypex = kerneltype; kerneltypey = kerneltype;
end

if length(h) == 2,
   hx = h(1); hy = h(2);
else
   hx = h; hy = h;
end

if length(degree) == 2,
   px = degree(1); py = degree(2);
else
   px = degree; py = degree;
end

switch kerneltypex
case {'Gaussian','gaussian','gauss','Gauss'}
   kernelsupportx = Inf;
otherwise
   kernelsupportx = hx;
end

switch kerneltypey
case {'Gaussian','gaussian','gauss','Gauss'}
   kernelsupporty = Inf;
otherwise
   kernelsupporty = hy;
end

zk = zeros(nrows,neval);
sk = zeros(size(x));
for i = 1:neval,
   io = (abs(x0-x(i))<kernelsupportx & abs(y0-y(i))<kernelsupporty);
   io = find(io);
   if ~isempty(io),
      xo = x0(io); yo = y0(io);
      jo = find(max(abs(z0(1:nrows,io)),[],2)>eps);
      wxo = evalkernel(x(i)-x0(io),hx,kerneltypex);
      wyo = evalkernel(y(i)-y0(io),hy,kerneltypey);
      zo = z0(jo,io);
      no = length(xo);
      if px*py > no,
         pmax = max(px,py); pmin = min(px,py);
         if pmin*pmin > no,
            if py>px,
               px = floor(sqrt(no));
               py = floor(no/px);
            else
               py = floor(sqrt(no));
               px = floor(no/px);
            end
         else
            px = min(px,floor(no/pmin));
            py = min(py,floor(no/pmin));
         end
      end
      zk(jo,i) = weightedLSpol2D(x(i),y(i),xo,yo,zo,wxo.*wyo,px,py);
   end
end
