
function [F lambda QQ gamma] = prefilterorthnotsparseSTCO(x,vanmom,e,gamma,nQ);

% prefilterorthnotsparse -- finds orthogonal prefilter for use in FMLPT, 
%                           not sparse, using an iterative scheme
%  Usage
%    [F lambda QQ gamma] = prefilterorthnotsparseSTCO(x,vanmom,nQ);
%    [F lambda QQ gamma] = prefilterorthnotsparseSTCO(x,vanmom,gamma);
%    [F lambda QQ gamma] = prefilterorthnotsparseSTCO(x,vanmom,e);
%    [F lambda QQ gamma] = prefilterorthnotsparseSTCO(x,vanmom,e,gamma,nQ);
%  Inputs
%    x          real vector
%    vanmom     integer, number of preserved moments
%    e          integer vector, entries into x; x(e) is the subsampled vector
%    gamma      real scalar, smaller or equal to one
%    nQ         integer, number of iterations
%  Outputs
%    F          ne x n matrix, where ne = length(e) and n = length(x)
%    lambda     eigenvalues (see paper)
%    QQ         cell array containing matrices of iteration steps
%    gamma      updated value of gamma
%  Description
%    The variables e,gamma,nQ may appear in arbitrary order
%    F*F' should be close to gamma*eye(ne)
%  Note
%    reference: Jansen & Amghar, Statistics and Computing
%  See also
%    help minQTAplusBvec
%    help constructprefilter
%    help prefiltbyvanmom


N = length(x);
x = column(x);

if nargin < 3, e = NaN; end
if nargin < 4, gamma = NaN; end
if nargin < 5, nQ = NaN; end

e1 = 1; elast = NaN;
if length(e)==1,
   if length(gamma) > 1, ebis = gamma; gamma = e; e = ebis;
   elseif length(nQ) > 1, ebis = nQ; nQ = e; e = ebis;
   else
      nQ = gamma; gamma = e;
      e = splitevenodd(x);
      Ne = length(e);
      F = zeros(Ne,N); F(1:Ne,e) = eye(Ne);
      % In the end points of x, no variance reduction is possible. We now take
      % off these points from further calculations. In these points F will
      % therefore behave is simple subsampling
      No = N-Ne;
      e1 = 1+floor(vanmom/2);
      elast = No-floor((vanmom-1)/2);
      if vanmom==0, elast = Ne; end
      % this is equivalent to: elast = min(Ne,elast);
      e = e(e1:elast); 
   end
end
Ne = length(e); if isnan(elast), elast = Ne; end
% variance reduction and higher vanishing moments cannot be satisfied in the
% end points of x (even for vanmom = 1, i.e., the Haar case, this is not
% possible, unless N is even)

if gamma >= 1,
   if isnan(nQ), nQ = gamma; gamma = NaN;
   elseif nQ <= 1 | isnan(nQ), nQbis = gamma; gamma = nQ; nQ = nQbis;
   else, error('parameter gamma must be 1 or lower'),
   end
end
if isnan(gamma),
switch vanmom
case 0,
  gamma = 1;
case 1,
  gamma = Ne/N;
case 2,
  xi = (mean(x(e))-mean(x))/sqrt(mean(x.^2)-mean(x)^2);
  zeta = 1-mean((x(e)-mean(x)).^2)/(mean(x.^2)-mean(x)^2);
  gamma = 1+sqrt(zeta^2+4*xi^2)/2-zeta/2;
  gamma = gamma*Ne/N;
end
end
if isnan(nQ), nQ = 40; end

J = zeros(Ne,N); J(1:Ne,e) = eye(Ne);
X = ones(N,vanmom);
for k = 1:vanmom-1, X(1:N,k+1) = x.^k; end
V = null(X')';
Nminvanmom = size(V,1); % Nminvanmom = N-vanmom;
Ve = V(1:Nminvanmom,e);
Ie = eye(Ne);
[E Lambda] = eig(Ve'*Ve-(1-gamma)*Ie);
%%%%%%%%%
% the first and last element of the evens has been left out of consideration
% for the sake of the 2 vanishing moments. As a result, the evens under
% consideration are less than half of the null space of the power function
% vector X.
% The more vanishing moments, the smaller the null space, and so the more evens
% (rows of F) have to be taken out from the inner construction.
% Lambdasqrt = zeros(Ne,Nminvanmom);
% Lambdasqrt(1:Ne,e) = sqrt(max(0,Lambda));
Lambdasqrt = sqrt(max(0,Lambda));
% Sjtilde = [E*Lambdasqrt*E' zeros(Ne,Nminvanmom-Ne)];
Sjtilde = E*Lambdasqrt*E';
% Sjtilde has Ne columns of length Ne. 
% Ve' has Nminvanmom>>Ne columns of length Ne. We
% now insert zero columns into Sjtilde so that Sjtilde has the same size as
% Ve', which is Ne x Nminvanmom.
% The Nminvanmom-Ne zero columns of Ne rows are inserted such
% that the existing columns in Sjtilde have maximal overlap with the
% corresponding column in Ve'. The insertion operation, Qjtilde0, is orthogonal.
[dummy jmax] = max(abs(Ve*Sjtilde));
Qjtilde0 = zeros(Ne,Nminvanmom);
Qjtilde0((jmax-1)*Ne+(1:length(jmax)))= 1;
Sjtilde = Sjtilde*Qjtilde0;

bandwidth = 2*vanmom+4;
weight = zeros(size(Qjtilde0));
l = floor((bandwidth-1)/2);
r = ceil((bandwidth-1)/2);

if length(weight) > 0, for d = -l:r, weight = weight+frwshift(Qjtilde0,d); end, end

[Q U it QQ] = minQTAplusBvec(Sjtilde',-Ve,1-weight',nQ,1); U = U';

F(e1:elast,1:N) = U*V+J;
QQ = {Qjtilde0,QQ{:}};

lambda = diag(Lambda);
