
function [Cp k] = CponSmodelsize(S,Y,X,stdev,t);

%  CponSmodelsize -- Mallows' Cp for LS or shrinkage estimates on selection S
%                    using models size as number of degrees of freedom
%  usage
%    [Cp k] = CponSmodelsize(S,Y,X,stdev,t);
%  Inputs
%    S      cell variable or integer vector, describing the successive models
%    Y      vector of size n, observed respons
%    X      design matrix, size n x m
%    stdev  positive real number
%    t      scalar or vector of size(S), threshold(s) used in lasso-estimation
%           default is zero, meaning: LS projection onto subsequent subsets of S
%  Outputs
%    Cp     Cp values for all selections S(1:s), s = 1,...,length(S)
%    k      number of nonzero parameters (i.e., sizes of active sets)
%  Description
%  Note
%    The variable S may be the output of LARS or LARSmodified.
%    If S is an integer vector, it is the ordered set of selected covariates.
%    negative values correspond to covariates that were first selected, then
%    taken out again.
%    If S is a cell variable, its entries are sets of selected covariates.
%  See also:
%    help LARS
%    help LARSell0
%    help LARSmodified 
%    help GCVonSmodelsize
%    help samplePEonS.m

if nargin<5, t = 0; end
if nargin<4, stdev=0; end
n = length(Y);
if min(size(X))==1, t=stdev; stdev=X; X = sparse(eye(n)); end
nS = length(S);
nt = length(t); if nt < nS, t = [row(t) t(nt)*ones(1,nS-nt)]; end
if size(Y,1)==1, Y=Y'; end
% k  = cumsum(sign(S));
A = []; SSe = zeros(1,nS); % A is set of active covariates
for s = 1:nS,
   if iscell(S), A = S{s};
   else, i = S(s); if i > 0, A = [A i]; else, A = setdiff(A,-i); end
   end
   k(s) = length(A);
   XA = X(1:n,A); muhat = XA*((XA'*XA)\ST(XA'*Y,t(s)));
   SSe(s) = norm(Y-muhat)^2;
end
Cp = SSe/n+2*k/n*stdev^2-stdev^2;
