
function LL = LLmargmixLaplnormplusnormal(pp,aa,ttau,stdev,y);

% LLmargmixLaplnormplusnormal -- marginal log-likelihood for mixture
%      p*Laplace+(1-p)*normal + normal noise
%   Usage
%      LL = LLmargmixLaplnormplusnormal(pp,aa,ttau,stdev,Y);
%      LL = LLmargmixLaplnormplusnormal(pa,stdev,Y);
%   Inputs
%      Y     observations
%      pp    sparsity parameter value
%      aa    parameter Laplace density
%      ttau  parameter normal density
%      pa    [pp aa ttau] or [pp; aa; ttau]
%      stdev value for st.dev. of noise
%   Outputs
%      LL    marginal log likelihood of parameters given Y
%   Description
%      The model is Y = X + stdev*Z, with
%      p = P(V =/= 0)
%      V|V=/=0 ~ Laplace(aa)
%      V|V=0 ~ NID(0,ttau^2)
%      Z ~ NID(0,1)
%   Note
%      ttau is assumed to be a common parameter for all components of Y: 
%           if it is a vector, then the log likelihood is evaluated in every of
%           its components
%      stdev is assumed to be a nuisance parameter. If it is a vector, then its
%      size must be that of Y, and every component of Y is assumed to have its
%      own value of stdev
%   See also
%      help LLmargmix2Laplplusnormal (mixture of two double exp.)
%      help LLmarg0inflLaplaceplusnormal (special case for ttau=0)

if nargin < 4,
   y = aa;
   stdev = ttau;
   pa = pp;
   [s1 s2] = size(pa);
   if s2 == 3,
      pp = pa(:,1); ttau = pa(:,2); aa = pa(:,3);
   elseif s1 == 3,
      pp = pa(1,:); ttau = pa(2,:); aa = pa(3,:);
   else
      error('inconsistent input variables')
   end
end
ntau = length(ttau); na = length(aa); np = length(pp);
nmax = max([ntau na np]);
if ntau < nmax,
   ttau = ttau(1)*ones(1,nmax); aa = row(aa); pp = row(pp);
end
if na < nmax,
   aa = aa(1)*ones(1,nmax); ttau = row(ttau); pp = row(pp);
end
if np < nmax,
   pp = pp(1)*ones(1,nmax); ttau = row(ttau); aa = row(aa);
end
np = length(pp); 
LL = zeros(size(pp));
for i = 1:np, 
   p = pp(i);
   tau = ttau(i);
   a = aa(i);
   taustdev = sqrt(tau^2+stdev.^2);
   LL(i) = sum(log(p*laplaceplusnormalpdf(y,a,stdev) + ...
                   (1-p)*gauss(y,taustdev)));
end
