
function [betahat, muhat, S, Cp, thr, Aopt] = LARSell0(Y,X,stdev,kmax)

% LARSell0 - Least Angle Regression for LASSO with ell0-based stopping
% criterion (including mirror-correction) 
%  Usage
%    [betahat muhat S Cp thr Aopt] = LARSell0(Y,X,stdev,kmax)
%  Inputs
%    Y      observations in sparsity model Y = X*beta+noise
%    X      model of covariates (rectangular matrix)
%    stdev  (estimated) standard deviation (default is 1; current version does
%           not estimate stdev)
%    kmax   maximum number of nonzeros (default is samplesize n=length(Y))
%  Outputs
%    betahat estimator beta, as provided by LARS (*)
%    muhat   estimator mu = X*beta
%    S       ordered set of selected variables (indices in {1,..,length(beta)})
%            negative indices point at taken out variables
%    Cp      Mallows' Cp values of nested selections, mirror-corrected
%    thr     threshold values used in computation of Cp-correction
%    Aopt    set of selected variables leading to minimum corrected Cp value
%  Description
%  Note
%     see p.74 in Elements of statistical learning, second edition;
%     downloadable from http://www-stat.stanford.edu/~tibs/ElemStatLearn/
%     see also paper LARS, Ann.Stat.2004, pages 407-499
%     This is a modified version of LARS, using parameter gammatilde
%         (3.6), p. 417, paper LARS, Ann.Stat.2004
%  (*) The estimator betahat is the LARS, shrunk estimator, corresponding to
%  the best least squares estimator after a variable selection without
%  shrinkage (i.e., using an ell0-stopping criterion). So, betahat itself is
%  NOT this best least squares estimator. That estimator can be found by
%  XA = X(1:n,Aopt);
%  betahatLS = (XA'*XA)\(XA'*Y);
%  See also:
%    help iterativeST

if nargin < 4, kmax = n; end
if nargin < 3, stdev = 1; end

Y = column(Y);

ybar = mean(Y);
Y = Y-ybar;
muhat = zeros(size(Y));
n = length(Y);
m = size(X,2);
colnormX = sqrt(sum(X.^2,1));
colnormX(colnormX<eps)=1;
X = X*diag(1./colnormX);
XTX = X'*X;

k = 0;
dof = k;
% Cp = [Cp(-1) Cp(0) Cp(1)]
Cp = Inf;
Cpmin = Inf;
muhatopt = zeros(size(Y)); Aopt = []; opt = [];
S = []; A = []; % A is the active set; S keeps track of the selection history
                % (with negative values for unselected variables)
Aprime = (1:m); % Complement of A in {1,2,...,m}
thr = [];
% while Cp(i+1) > Cp(i+2),
kmax = min(kmax,m-1);
while k < kmax+1,
disp(['k = ' num2str(k) '; still smaller than kmax = ' num2str(kmax)])
   muhat0 = muhat;
   if k>0,
      schatA = sign(chat(A));
      XA = X(1:n,A)*diag(schatA);
      GA = XA'*XA;
      AA = 1/sqrt(ones(1,k)*(GA\ones(k,1)));
      wA = AA*(GA\ones(k,1));
      uA = XA*wA;
      epseig = 1.e-12;
      if min(eig(GA))<epseig,
         % GAinv = pinv(GA,0.01);
         % pinv is not very stable
         threig = 1.e-2;
         [V L] = eig(GA);
         L = diag(L);
         Linv = zeros(size(L));
         Labovethreig = find(abs(L)>threig);
         Linv(Labovethreig) = 1./L(Labovethreig);
         GAinv = V*diag(Linv)*V';
         AA = 1/sqrt(ones(1,k)*(GAinv*ones(k,1)));
         wA = AA*(GAinv*ones(k,1));
         uA = XA*wA;
      end
   else
      chat = X'*Y; [Chat, j] = max(abs(chat));
      % Chat is the value of lambda when activating the first component (but
      % keeping it at zero for the moment)
      uA = X(1:n,j); uA = uA/norm(uA);
      AA = 1;
   end
   a = X'*uA;
   gamma = [(Chat-chat(Aprime))./(AA-a(Aprime)), ...
            (Chat+chat(Aprime))./(AA+a(Aprime))];
   gamma(gamma<0) = Inf;
   gamma = min(gamma,[],2);
   [gammahat, j] = min(gamma);
   gammatilde = Inf;
   if k>0,
      dS = schatA.*wA;
      gammatilde = -betahatS./dS;
      gammatilde(gammatilde<eps) = Inf;
      [gammatilde jout] = min(gammatilde);
   end
   if gammatilde < gammahat,
      % Modified LARS: See (3.6), p. 417, paper LARS, Ann.Stat.2004
      muhat = muhat0 + gammatilde*uA;
      newA = [A(1:jout-1) A(jout+1:k)];
      jout = A(jout); Aprime = [Aprime jout]; A = newA;
      S = [S -jout];
      k = k-1;
      gammahat = gammatilde;
   else
      muhat = muhat0 + gammahat*uA;
      newAprime = [Aprime(1:j-1) Aprime(j+1:m-k)];
      j = Aprime(j); A = [A j]; Aprime = newAprime;
      S = [S j];
      k = k+1;
   end
   dof = k;
   Chatnew = Chat -gammahat*AA;
   chat = X'*(Y-muhat); chatA = chat(A); Chat = max(abs(chatA));
   % Chat is the value of lambda that is about to activate the k-th=(dof+1)st
   % component (but keeping it at value 0 for the moment)
   % Check: compare two calculations of Chat==Chatnew
   check = Chat-Chatnew; tol = 1.e-10;
   if abs(check)>tol, warning('inconsistency in computations of lambda'), end
   % Now store Chat in lambda
   % Cp = [Cp norm(Y-muhat0)^2/n+2*dof/n*stdev^2-stdev^2];
   lambda = Chat;

   XA = X(1:n,A);
   XAprime = X(1:n,Aprime);
   XTXA = XA'*XA;
   ProjS = XA*(XTXA\XA');
   % we compute (P-I)*X_{S'}
   % columns of this projection that are small are set to zero and the
   % corresponding variables are excluded from future selection
   PminIXAprime = ProjS*XAprime-XAprime;
   PminIXAprime0 = find(abs(sum(PminIXAprime,1))<n*eps);
   X(1:n,Aprime(PminIXAprime0)) = 0;
   XAprime = X(1:n,Aprime);
   chat(Aprime(PminIXAprime0)) = 0;
   % We compute X_{S'}^TX_{S'} and its pseudo-inverse. The pseudo-inverse is
   % computed with eigenvalues (as the matrix is symmetric and the matlab
   % function pinv is found to be too sensitive to numerical errors)
   XTXAprime = XAprime'*XAprime;
   XTXAAprime = XA'*XAprime;
     threig = 1.e-2;
     [V L] = eig(XTXAprime);
     L = diag(L);
     Linv = zeros(size(L));
     Labovethreig = find(abs(L)>threig);
     Linv(Labovethreig) = 1./L(Labovethreig);
     XTXAprimeinv = V*diag(Linv)*V';
   B = XTXAprimeinv*XTXAAprime';
   % next line computes Schur complement of X_S^T*X_S within X^T*X
   % This is one of the two terms in the computation of
   % cov(eta_S|variables in S' do not meet selection criterion)
   XTXAgivenXAprime = XTXA - XTXAAprime*B;
   % the second term in
   % cov(eta_S|variables in S' do not meet selection criterion)
   % is
   % cov(E(eta_S|variables in S' do not meet selection criterion))
   % This covariance follows from
   % B*cov(E(eta_{S'}|variables in S' do not meet selection criterion))*B'
   % with B = (X_{S'}^T*X_{S'})^{-1}*(X_S^T*X_{S'})
   % For the computation of
   % cov(E(eta_{S'}|variables in S' do not meet selection criterion))
   % we assume that if E are the eigenvectors for
   % cov(E(eta_{S'}) = stdev^2*(X_{S'}^T*X_{S'})
   % then the same eigenvectors remain for the conditional covariance
   % in other words: the selection criterion can be expressed in terms of the
   % principal components
   [E Lambda] = eig(XTXAprime); Lambda = max(0,Lambda);
   stdev2Lambda = stdev^2*diag(Lambda);
   stdevLambda = sqrt(stdev2Lambda);
   covAprime = stdev2Lambda - ...
               2*stdev2Lambda*lambda.*gauss(lambda,stdevLambda)*(n/m)./...
               (1-(n/m)*2*(1-cumgauss(lambda,stdevLambda)));
   covAprime = E*diag(covAprime)*E';
   covEAgivenAprime = B'*covAprime*B;
   betahatS = XTXA\(XA'*muhat);
   betahatH = XTXA\(XA'*Y);
   betahat = zeros(m,1);
   betahat(A) = betahatH;
   muhatH = XA*betahatH;
   [E Lambda] = eig(covEAgivenAprime+stdev^2*XTXAgivenXAprime);
   stdev2Lambda = diag(real(Lambda));
   stdevLambda = sqrt(max(0,stdev2Lambda));
   mirr = 4*stdev2Lambda*lambda.*gauss(lambda,stdevLambda);
   Cpk = norm(Y-muhatH)^2/n+2*stdev^2*k/n-stdev^2 + mean(mirr);
   if Cpk < Cpmin,
      Cpmin = Cpk;
      Aopt = A(1:k);
      muhatopt = muhat;
   end
   Cp = [Cp, Cpk];
   thr = [thr, mean(lambda)];
end
muhat = muhatopt+ybar;
XAopt = X(1:n,Aopt);

% HT
% betahatAopt = (XAopt'*XAopt)\(XAopt'*Y);
% ST
betahatAopt = (XAopt'*XAopt)\(XAopt'*muhatopt);
betahat = zeros(1,m);
betahat(Aopt) = betahatAopt;

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
