
function [betahat, muhat, S, Cp, lambda, Aopt] = LARSmodified(Y,X,stdev,k0)

% LARSmodified - Least Angle Regression for LASSO 
%  Usage
%    [betahat muhat x Cp lambda Aopt] = LARSmodified(Y,X,stdev,k0)
%  Inputs
%    Y      observations in sparsity model Y = X*beta+noise
%    X      model of covariates (rectangular matrix)
%    stdev  (estimated) standard deviation (default is 1; current version does
%           not estimate stdev)
%           if stdev==NaN, then the stop criterion is based on GCV instead of
%           Mallows's Cp
%    k0     desired number of nonzeros (default is n=length(Y))
%  Outputs
%    betahat estimator beta
%    muhat   estimator mu = X*beta
%    S       ordered set of selected variables (indices in {1,..,length(beta)})
%            negative indices point at taken out variables
%    Cp      Mallows's Cp (or GCV) values of nested selections
%    lambda  values of penalty parameter corresponding to S and Cp
%    Aopt    set of selected variables leading to minimum Cp value
%  Description
%  Notes
%     (1) see p.74 in Elements of statistical learning, second edition;
%     downloadable from http://www-stat.stanford.edu/~tibs/ElemStatLearn/
%     see also paper LARS, Ann.Stat.2004, pages 407-499
%     This is a modified version of LARS, using parameter gammatilde
%         (3.6), p. 417, paper LARS, Ann.Stat.2004
%     (2) LARSmodified assumes the columns of X to be normalised.
%     Set: D = diag(sqrt(sum(X.^2))); X = X/D; 
%     (Note that this normalisation is different than that in iterativeST.m)
%  See also:
%    help iterativeST

if nargin < 4, k0 = n; end
if nargin < 3, stdev = 1; end

Y = column(Y);

ybar = mean(Y);
Y = Y-ybar;
muhat = zeros(size(Y));
n = length(Y);
m = size(X,2);
colnormX = sqrt(sum(X.^2,1));
X = X*diag(1./colnormX);

k = 0;
dof = k;
% Cp = [Cp(-1) Cp(0) Cp(1)]
Cp = Inf;
Cpmin = Inf;
muhatopt = zeros(size(Y)); Aopt = [];
lambda = Inf;
S = []; A = []; % A is the active set; S keeps track of the selection history
                % (with negative values for unselected variables)
Aprime = (1:m); % Complement of A in {1,2,...,m}
% while Cp(k+1) > Cp(k+2),
k0 = min(k0,m-1);
while k < k0+1,
   muhat0 = muhat;
   if k>0,
      schatA = sign(chat(A));
      XA = X(1:n,A)*diag(schatA);
      GA = XA'*XA;
      AA = 1/sqrt(ones(1,k)*(GA\ones(k,1)));
      wA = AA*(GA\ones(k,1));
      uA = XA*wA;
      epseig = 1.e-12;
      if min(eig(GA))<epseig,
         % GAinv = pinv(GA,0.01);
         % pinv is not very stable
         threig = 1.e-2;
         [V L] = eig(GA);
         L = diag(L);
         Linv = zeros(size(L));
         Labovethreig = find(abs(L)>threig);
         Linv(Labovethreig) = 1./L(Labovethreig);
         GAinv = V*diag(Linv)*V';
         AA = 1/sqrt(ones(1,k)*(GAinv*ones(k,1)));
         wA = AA*(GAinv*ones(k,1));
         uA = XA*wA;
      end
   else
      chat = X'*Y; [Chat, j] = max(abs(chat));
      % Chat is the value of lambda when activating the first component (but
      % keeping it at zero for the moment)
      uA = X(1:n,j); uA = uA/norm(uA);
      AA = 1;
   end
   a = X'*uA;
   gamma = [(Chat-chat(Aprime))./(AA-a(Aprime)), ...
            (Chat+chat(Aprime))./(AA+a(Aprime))];
   gamma(gamma<0) = Inf;
   gamma = min(gamma,[],2);
   [gammahat, j] = min(gamma);
   gammatilde = Inf;
   if k>0,
      dA = schatA.*wA;
      gammatilde = -betahatA./dA;
      gammatilde(gammatilde<eps) = Inf;
      [gammatilde jout] = min(gammatilde);
   end
   if gammatilde < gammahat,
      % Modified LARS: See (3.6), p. 417, paper LARS, Ann.Stat.2004
      muhat = muhat0 + gammatilde*uA;
      newA = [A(1:jout-1) A(jout+1:k)];
      jout = A(jout); Aprime = [Aprime jout]; A = newA;
      S = [S -jout];
      k = k-1;
      gammahat = gammatilde;
   else
      muhat = muhat0 + gammahat*uA;
      newAprime = [Aprime(1:j-1) Aprime(j+1:m-k)];
      j = Aprime(j); A = [A j]; Aprime = newAprime;
      S = [S j];
      k = k+1;
   end
   XA = X(1:n,A);
   betahatA = (XA'*XA)\(XA'*muhat);
   dof = k-1;
   Chatnew = Chat-gammahat*AA;
   chat = X'*(Y-muhat); chatA = chat(A); Chat = max(abs(chatA));
   % Chat is the value of lambda that is about to activate the k-th=(dof+1)st
   % component (but keeping it at value 0 for the moment)
   % Check: compare two calculations of Chat==Chatnew
   check = Chat-Chatnew; tol = 1.e-10;
   if abs(check)>tol, warning('inconsistency in computations of lambda'), end
   % Now store Chat in lambda(dof+2) 
   lambda = [lambda, Chat];
   if isnan(stdev), % use GCV
      Cpk = (norm(Y-muhat)^2/n)/(1-dof/n)^2;
   else
      Cpk = norm(Y-muhat)^2/n+2*dof/n*stdev^2-stdev^2;
   end
   if Cpk < Cpmin,
      Cpmin = Cpk;
      Aopt = A(1:dof);
      muhatopt = muhat;
   end
   Cp = [Cp, Cpk];
end
muhat = muhatopt+ybar;
A = Aopt;
XA = X(1:n,A);

% HT
% betahatA = (XA'*XA)\(XA'*Y);
% ST
betahatA = (XA'*XA)\(XA'*muhatopt);
betahat = zeros(1,m);
betahat(A) = betahatA;

% Copyright (c) Maarten Jansen
%
% This software is part of ThreshLab and is copyrighted material.
