
function [wt, thr] = minmsethreshest(v,w,softNOThard,varargin)

% minmsethreshest -- apply soft/hard threshold with minimum MSE threshold
%                    MSE = Loss = mean squared error
%  Usage
%    [wt, thr] = minmsethreshest(v,w,soft,'maxthr',maxthr,'minthr',minthr);
%  Inputs
%    v            noise-free input vector
%    w            observed input vector
%    soft         boolean: true for soft; false for hard
%    maxthr       maximum possible threshold (optional)
%    minthr       minimum possible threshold (optional)
%  Outputs
%    wt     thresholded vector
%    thr    minimum MSE threshold
%  Description
%    Applies minimum MSE soft- or hard threshold to all elements of w 
%  Note
%    For making a plot of mse for a given vector of thresholds
%    use MSE1thresh (for one dimension)
%
%  See also
%    MSEthresh

if nargin<3, softNOThard = false; end

s = 1;
if isnumeric(softNOThard) == true,
   if length(softNOThard) == length(v), 
      s = softNOThard; softNOThard = false;
   end
end

N = size(w,1)*size(w,2);
wt = w;

% identification of maximum and minimum threshold
% default values:
maxthr = max(abs(reshape(w,N,1)));
minthr = 0;
nvarargin = length(varargin);
for k = 1:nvarargin, vark = varargin{k};
   if ischar(vark),
      switch vark(1:6),
      case {'maxthr','thrmax'},
         maxthr = varargin{k+1};
         minthr = maxthr/m;
      case {'minthr','thrmin'},
         minthr = varargin{k+1};
      case {'1','S','ST','s','st','soft','Soft',...
            '0','H','HT','h','ht','hard','Hard'},
         softNOThard = vark;
      end
   elseif islogical(vark),
      softNOThard = vark;
   elseif isnumeric(vark),
      if ismember(vark,[0 1]), softNOThard = vark; end
   end
end

if ischar(softNOThard),
   switch softNOThard,
   case {'1','S','ST','s','st','soft','Soft'},
      softNOThard = true;
   case {'0','H','HT','h','ht','hard','Hard'},
      softNOThard = false;
   otherwise
      softNOThard = true;
   end
end

means = mean(s);
s = s/means;

%initialisation Fibonacci numbers
F0 = 1;
F1 = 1;
F2 = 2;
n_iter = 1;
a = minthr*s;
b = maxthr*s;

% Computation of needed number of iteration steps
eps = 0.0001;
while norm(b-a)/F2 > eps
  F0 = F1;
  F1 = F2;
  F2 = F0 + F1;
  n_iter = n_iter + 1;
end

% minimization routine

r = a + (F1/F2) * (b - a);
fr = MSEthresh(v,w,r,softNOThard); 
l = a + (F0/F2) * (b - a);
fl = MSEthresh(v,w,l,softNOThard);
     
for k = 1:n_iter - 1
   if (fl > fr)
      a = l;
      l = r;
      fl = fr;
      r = b-l+a;
      fr = MSEthresh(v,w,r,softNOThard);
   else
      b = r;
      r = l;
      fr = fl;
      l = a+b-r;
      fl = MSEthresh(v,w,l,softNOThard);
   end
end
     
thr = b;

if softNOThard==true,
   wt = ST(w,thr);
else
   wt = HT(w,thr);
end

if max(size(thr))>1, thr = mean(b)/means; end

% Copyright (c) Maarten Jansen
% 
% This software is part of ThreshLab and is copyrighted material. 
