
function mirror = empiricalmirror(S,X,noise);

% empiricalmirror - ThreshLab/Threshold/Risk/ 
%   Usage
%     mirror = empiricalmirror(S,X,noise); 
%     mirror = empiricalmirror(S,X);
%   Inputs
%     S      vector of integers; maximum selection
%     X      n times m design matrix  in the linear model Y=X*beta+stdev*Z
%     noise  (stdev*Z) n-vector of reals
%   Outputs
%     mirror vector of mirror effects (standardized or not, see below)
%   Description
%     computes empirical mirror, where (in expected value)
%              mirror = (dof-k)*stdev^2/n 
%                     = {E[norm(P_Sk*Z)]-k}*stdev^2/n + o[PE(k)] as n->Inf
%              where P_Sk is the orthogonal prediction onto the columns X(:,Sk)
%     for selections Sk with size #Sk = k 
%     The output of this routines is
%     (1) either the exact realisation of [norm(P_Sk*Z)-k]*stdev^2/n, 
%     for all Sk=S(1:s), with k from 1 to length(S),
%     if the input consists of S,noise and X
%     (This requires the exact noise to be known)
%     (2) or the standardised, bootstrap realisation [norm(P_Sk*Zstar-k]/n
%     for all Sk that sort from LARSmodified(Zstar,X,1,kmax), with kmax the
%     maximum size of models in S
%     if the input consists of S,X
%     (This is the more realistic situation)
%   Note
%     if the input consists of X,kmax, the size vector k may contain zeros!

if min(size(S))>1, % S is actually X;
   Scopy = X; X = S; S = Scopy;
end
n = size(X,1);
k = cumsum(sign(S));
if nargin > 2, % exact empirical mirror
   stdev = sqrt(var2(noise));
   Zstar = noise/stdev; Sstar = S;
else % bootstrap/simulated mirror
   stdev = 1;
   randn('state',0); 
   Zstar = randn(n,1);
   [beta0hat mu0hat Sstar] = LARSmodified(Zstar,X,1,max(k));
end
nSstar = length(Sstar);
kstar = cumsum(sign(Sstar));
kstarmax = max(kstar); Sfork = zeros(1,kstarmax);
ii = []; normPZstar = zeros(1,nSstar);
for s = 1:nSstar,
   i = Sstar(s);
   if i > 0, ii = [ii i]; else, ii = setdiff(ii,-i); end
   Xii = X(1:n,ii); PZ = Xii*((Xii'*Xii)\(Xii'*Zstar));
   normPZstar(s) = norm(PZ)^2;
   if kstar(s)>0, Sfork(kstar(s)) = s; end
end
mirrorSstar = (normPZstar-kstar)/n*stdev^2;
mirrorkstar = mirrorSstar(Sfork);
nonzero = find(k>0);
mirror(nonzero) = mirrorkstar(k(nonzero));

