
function [thr PE] = minriskthr(beta,stdev);

% minriskthr -- finds soft threshold that minimizes expected L2-loss 
%  Usage
%    thr = minriskthr(beta,stdev);
%  Inputs
%    beta   vector of noise-free data
%    stdev  standard deviation of normal noise (to be added to beta)
%  Outputs
%    thr    optimal threshold w.r.t. expected L2-loss
%  Description
%    finds soft threshold that minimizes expected L2-loss in thresholding 
%    Y = beta+noise where noise ~ N(0,stdev^2)
%    This implementation is based on fixed point iteration
%  See also
%    help minrisksoftthr: another implementation for the same problem


if nargin == 1, stdev = 1; end
eps = 1.e-10;
maxiter = 5000;
thmax = max(abs(beta));
n = length(beta);
t1 = max(1,sqrt(abs(2*log(n))))*stdev;
t0 = 0;
iter = 0;
while (abs(t0 - t1) > eps)
   t0 = t1;
   iter = iter + 1;
   numerator = gauss(t1 - beta, stdev) + gauss(t1 + beta,stdev);
   denominator = Rcumgauss(t1 - beta,stdev) + Rcumgauss(t1 + beta,stdev);
   if (iter < maxiter)
      t1 = stdev^2*sum(numerator)/sum(denominator);
   end    
end
thr = t1;
PE = softrisk(t1,beta,stdev);

% Copyright (c) Maarten Jansen
% 
% This software is part of ThreshLab and is copyrighted material. 
