
function [PE k] = samplePEonS(S,Y,X,beta,t);

% samplePEonS -- sample pred. err. for LS or shrinkage est. on selection S
%  usage
%    PE = samplePEonS(S,Y,X,beta,t);
%    PE = samplePEonS(S,Y,mu
%  Inputs
%    S      cell variable or integer vector, describing the successive models
%    Y      vector of size n, observed respons
%    X      design matrix, size n x m
%    beta   vector of size m, true parameter values
%    t      scalar or vector of size(S), threshold(s) used in lasso-estimation
%           default is zero, meaning: LS projection onto subsequent subsets of S
%  Outputs
%    PE     Sample prediction errors for all selections S(1:s), 
%           s = 1,...,length(S)
%    k      number of nonzero parameters (i.e., sizes of active sets in S(1:s))
%  Description
%  Note
%    The variable S may be the output of LARS or LARSmodified.
%    If S is an integer vector, it is the ordered set of selected covariates.
%    negative values correspond to covariates that were first selected, then
%    taken out again.
%    If S is a cell variable, its entries are sets of selected covariates.
%  See also:
%    help LARS
%    help LARSell0
%    help LARSmodified 
%    help GCVonSmodelsize
%    help CponSmodelsize

if nargin<4, beta=0; end
if nargin<5, t = 0; end
n = length(Y);
if min(size(X))==1, t=beta; beta=X; X = sparse(eye(n)); end
nS = length(S);
nt = length(t); if nt < nS, t = [row(t) t(nt)*ones(1,nS-nt)]; end
if size(Y,1)==1, Y=Y'; end, if size(beta,1)==1, beta=beta'; end
% k  = cumsum(sign(S));
A = []; PE = zeros(1,nS); % A is set of active covariates
mu = X*beta;
for s = 1:nS,
   if iscell(S), A = S{s};
   else, i = S(s); if i > 0, A = [A i]; else, A = setdiff(A,-i); end
   end
   k(s) = length(A);
   XA = X(1:n,A); muhat = XA*((XA'*XA)\ST(XA'*Y,t(s)));
   PE(s) = norm(mu-muhat)^2/n;
end
