
function [maxmas trees] = maxmassubtreeXL(x,parents,nnodes,compr,dummies,leafend)

% maxmassubtreeXL - ThreshLab/Trees - maxmassubtree for XL data sets
%   Usage
%     [maxmas trees] = maxmassubtreeXL(x,parents,nnodes);
%     [maxmas trees] = maxmassubtreeXL(x,children,nnodes);
%   Inputs
%     x        real vector, weight/mass in every node of the tree (see below)
%     parents  integer vector of size(x), indices into x, for each element of x
%              containing the index of its ancestor (default = floor((1:n)/2),
%              i.e., maximal/full binary tree)
%     children cell or integer matrix, indices into x (no default)
%     nnodes   integer larger than 2; maximum size of subtree (number of 
%              nodes; default = length(x))
%     compr    boolean; if true, then output trees is replaced by a compressed
%                       representation (to be unpacked by reconstructsubtree)
%     dummies  vector of integers, indices into x (default empty)
%              dummies are variables whose values do not count for selection
%              and whose selection does not count for the maxmass, nor for the
%              size of the selection (typically these variables are
%              pre-selected)
%     leafend  boolean (default false). If true, then admissible trees must
%                      not have end leaves with children in the full tree
%   Outputs
%     maxmas   vector with components sum(x(trees{i}))
%     trees    cell of n arrays with integer vectors
%              or (if compr = true), compressed representation
%   Description
%     This routine performs a backtracking search for optimal k-subtrees for
%     all values of k between 1 and nnodes.
%   See also
%     help maxmassubtree

n = length(x);
% n0 = n; n = 2^(ceil(log2(n+1)))-1; x0 = x; x = zeros(1,n); x(1:n0) = row(x0);
if nargin < 6, leafend = false; end
if nargin < 5, dummies = []; end
if nargin < 4, compr = false; end
if nargin < 3, nnodes = n; end
if nargin < 2, parents = floor((1:n)/2); end
if n>1,
   if max(size(nnodes)) == n,
      change = nnodes; nnodes = parents; parents = change;
   end
end
if nnodes < 1 | nnodes-floor(nnodes) ~= 0,
   disp(['nnodes = ' num2str(nnodes)])
   error('nnodes must be a positive integer')
end
if length(dummies)==n & all(ismember(dummies,[0 1])),
   isdummy = dummies; dummies = find(isdummy);
else
   isdummy = ismember(1:n,dummies);
end

% make sure to have parents as input
[parents children] = parentschildren(parents);
parentsfull = parents; % make a copy for use at the end of this routine

% add one superroot 
x = [0 row(x)]; parents = [0 parents+1]; nnodes = nnodes+1; n = n+1;
if ~isempty(dummies), dummies = dummies+1; end
isdummy = [true isdummy];

% erase values of dummies
x(dummies) = 0;

% Now, find the children
children = parentschildren(parents,'cell');
childrencopy = children;

nnodeswithc = length(children); % nodes beyond nnodeswithc cannot possibly 
                                % have children
                                % (other nodes may or may not have children)
cc = 1;
maxmas = {};
right = cell(1,n); % contains for each node and for each k which subtree rooted 
                   % in that node should be passed to the parent node to form
                   % the best k-subtree of that parent node. This cell variable
                   % is a comparatively compressed way of keeping track of the
                   % optimal subtrees throughout the search routine.
active = ones(1,n); nactive = n;
level = 0; % throughout the iterations, the routine will maintain a path from
           % the top (root) of the tree down to leaf level
while nactive>1,
   % go down in tree, as far/as deep as possible
   while length(cc)>0, 
      level = level+1; p = cc(1);
      maxmas = {maxmas{:},x(p)}; % adds a subtree rooted at p with just p as
                                 % single node (to be extended)
      if p>nnodeswithc, cc = []; else, cc = children{p}; end
   end
   % merge p and its parent (backtracking)
   % p and c may be single node trees, or roots of extended trees.
   c = p; p = parents(c);
   massp = maxmas{level-1}; ntp = length(massp); massc0 = [0 maxmas{level}];
   if leafend==true & ntp==1, massc0(1) = -Inf; end
   % if leafend==true, then merging empty c with end node p leads to an
   % inadmissible tree (and this is the only source of inadmissible trees) 
   if isdummy(c), massc0 = maxmas{level}; end 
   % if isdummy(c), then adding one node from subtree of c is like adding zero
   % nodes, so we can omit the zero nodes in front.
   % if isdummy(p), then this will be taken into account as soon as p has
   % become c.
   ntc = length(massc0)-1;
   ntpc = ntp+ntc; % ntpc maximum size of tree combining p and c
   masspc = zeros(1,ntpc); 
   rightc = zeros(1,ntpc);
   for s = 1:ntpc,
      % best s-subtree combining p with c. This subtree must contain at
      % least one node from p (the root p itself), otherwise, it cannot
      % be linked to nodes further up in the tree.
      s1 = max(1,s-ntc); s2 = min(ntp,s);
      addedmass = massp(s1:s2)+massc0(s-s1+1:-1:s-s2+1);
      [masspc(s) l] = max(addedmass); l = l+s1-1;
      rightc(s) = s-l;
   end
   % clean up after merging and go one level up in tree
   % cut off child (no longer needed), remove its entries in maxmas
   right{c} = rightc;
   cc = setdiff(children{p},c); children{p} = cc;
   active(c) = 0; nactive = nactive-1;
   level = level-1;
   maxmas{level} = masspc;
   maxmas = maxmas(1:level);
   % Now jump back to the beginning of the loop. 
   % From there, we may either redescend into another subtree, or we may have 
   % to climb up and merge further, before we can go down again.
end

disp('maxmassubtreeXL has found best subtrees')
maxmas = maxmas{1};
nnodes = nnodes-1;
nnodes = min(nnodes,length(maxmas)-1);
maxmas = maxmas(2:nnodes+1);
parents = parents(2:nnodes+1)-1;
dummies = dummies-1;

if compr==false,
   trees = reconstructsubtree(parentsfull,right,1:nnodes,dummies);
else
   trees = right;
end
   
% Copyright (c) Maarten Jansen
% 
% This software is part of ThreshLab and is copyrighted material. 
