
function KappaXhat = nodewisereggraph(X,S);

% nodewisereggraph - ThreshLab/Trees
%   Usage
%     KappaXhat = nodewisereggraph(X,S);
%   Inputs
%     X           m x n matrix; n observations of m-variate normal random
%                                                   vector with zero mean
%     S           binary m x m matrix, ones pointing to nonzeros in output
%   Outputs
%     KappaXhat   estimated concentration matrix
%   Description
%     applies nodewise regression method for the estimation of the
%     concentration matrix from observations of a multivariate normal RV.
%   Remarks
%     (1) Symmetry in the output matrix is not guaranteed.
%     Set KappaXhat = (KappaXhat+KappaXhat')/2; to impose symmetry
%     (2) Unless S = ones(m,m) (all ones; no selection), the output KappaXhat
%     is (most probably) not the maximum likelihood estimator of the
%     concentration matrix; see maxLsparseKappa
%   See also
%     help nodewiseLassoGCVCpmirror
%     help maxLsparseKappa

[m,n] = size(X);
if nargin<2, S = ones(m,m); end
if size(S,1)==2 & size(S,2)>2, S=S'; end
if size(S,2)==2 & size(S,1)>2, S1 = S(:,1)+m*(S(:,2)-1); S = S1; end
if min(size(S))==1, S0 = zeros(m,m); S0(S)=1; S = S0; end
S = min(1,S+S');
if sum(sum(S))<m*m/2,
   KappaXhat = sparse(zeros(m,m)); S = sparse(S);
else
   KappaXhat = zeros(m,m);
end

for c=1:m,
   notc = [1:c-1 c+1:m];
   Yc = X(c,1:n)'; Xc = X(notc,1:n)';
   s = find(S(c,notc));
   Xcs = Xc(1:n,s);
   betacshat = pinv(Xcs)*Yc;
   sigma2chat = norm(Yc-Xcs*betacshat)^2/n;
   KappaXhat(c,c) = 1/sigma2chat;
   KappaXhat(c,notc(s)) = -KappaXhat(c,c)*betacshat';
end
