function [c it] = finescaleBsplinecoefs(x,f,ptilde,regmethod,varargin);

% finescaleBsplinecoefs - ThreshLab2/Irregular/1D/ --- fine scale coefs from
%                                                      function values
% Usage
%    c = finescaleBsplinecoefs(x,f,ptilde,regmethod,...);
%    (see Examples below for typical use)
% Inputs
%    x           knots of the spline basis, vector of length n
%    f           function values in knots (can be a matrix)
%    ptilde      number of vanishing moments (ptilde-1 is degree of polynomials)
%    regmethod       text describing regularisation method (default = 'I'; see
%                    also below) 
%    'maxit',maxit   maximum number of iterations
%    'relax',relfac  relaxation factor
%    'c0',c0         initial solution; c0 can be vector or string (see below)
%    'itfilter',<string>  filter operation in Landweber iterations (see below)
%    'its2filter',<scalar or vector>   # of Landweber iterations between filter
%                                      applications (see below)
% Outputs
%    c       coefficients in B-spline-basis Phi(x) on knots x
%            (Phi(x) being a row of basis functions)
% Description
%    Finds c so that Phi(x)*c approximates (x,f), EXCEPT in first and last
%    knot. This is because Phi(x) is a zero row in the first and last knot.
%    Regularisation of the matrix Phi(x) (x being the vector of knots) is
%    carried out, using value of regmethod:
%    regmethod = 'derivative'
%         Phi(x)*c interpolates (x,f), EXCEPT in first and last knot, and c
%         by minimizes of the norm of the derivatives in the inside knots.
%    regmethod = 'LS','I'
%         Phi(x)*c interpolates (x,f), EXCEPT in first and last knot, and
%         norm(c-f) is minimized (least squares w.r.t. function values, i.e., 
%         find interpolating spline with coefficients as close as possible to
%         function values)
%    regmethod = 'pinv'
%         Phi(x)*c interpolates (x,f), EXCEPT in first and last knot, and
%         norm(c) is minimized (pseudo inverse of the system Phi(x)*c = f)
%    regmethod = 'Landweber'
%         cnew = c + relfac*Phi(x)'*[Phi(x)*c-f]
%    regmethod = 'Tikhonov', followed by a value of alpha (optional)
%         Tikhonov regularisation
%         c = Phi'*inv(Phi'*Phi + alpha*I)'*f
%    regmethod = 'powerLS','powerI'
%         least squares w.r.t. function values, as for regmethod = 'LS','I'
%         but here Phi(x)*c interpolates (x,f), only if f(x) is a polynomial of
%         degree ptilde-1 or less. 
%         This option is best for data with jumps and data with noise
%    'c0', c0  if c0 is a string 'I','LS','deriv', it describes a direct method
%              whose outcome will be used as initial vector for the iterative
%              part. c0 can also be a vector (outcome from previous function
%              call)
%    'itfilter',filtername    invokes the application of a filter to cnew
%                             before c gets the value of cnew
%    'its2filter',scalar      applies the filter only after a given+1
%                             (=scalar) number of Landweber steps 
%                             (default is 1)
%    'its2filter',vector      applies the filter only after the steps given in
%                             the vector
% Notes
%    (1)
%    The routine assumes that the first and last knots have a multiplicity 
%    to deal with all definitions involving knots with numbers outside
%    {1,...,n}
%    It is suggested to add two artificial knots outside but near both end
%    knots, with extrapolated or even arbitrary values in f. The end values
%    are never used in this routine (so they don't have any impact)
%    (2)
%    The objective of the routine is to fully exploit the benefits of the
%    approximative power of the B-spline representation: B-splines of degree
%    ptilde-1 reproduce polynomials up to degree ptilde-1, and thus have good
%    approximation results for functions that are well approximated by a
%    polynomial. If the function is not well approximated by a polynomial,
%    e.g., if the function contains a jump, then application of this routine is
%    useless. Even worse, it will show an unstable outcome. This is the case
%    for functions with jumps, and, a fortiori, for functions with noise.
% Examples
%    Check illustratefinescalecoefs.m for simulations with:
%    (1) Smooth function/signals
%    (2) Signals with jumps
%    (3) Data with errors (noisy signal)
% See Also
%     help Bsplinebasis
%     help derivBsplinebasis
%     type illustratefinescalecoefs

if nargin < 4, regmethod = 'I'; end
n = length(x);
sf = size(f);
dotranspose = false;
if ~ismember(sf(1),[n-2 n]),
   f = f'; dotranspose = true; sf = size(f);
end
m = sf(2);
if sf(1) == n-2,
   fbar = f;
elseif sf(1) == n,
   fbar = f(2:n-1,1:m);
else
   error('dimension mismatch inputs f and x')
end
Phi = sparse(Bsplinebasis(x,x,ptilde,false));
Phibar = Phi(2:n-1,1:n);

nvarargin = length(varargin);
vararginsentinel = {varargin{:},NaN};


switch lower(regmethod),
case {'derivative','deriv','i','ls','pinv'},
   initmethod = regmethod;
   itermethod = 'direct';
case {'landweber'},
   relaxfactor = NaN;
   maxit = 1000;
   c0 = NaN;
   itfilter = 'eye';
   its2filter = 1;
   ntaps = 3;
   tol = 1.e-6;
   for k = 1:nvarargin, vark = varargin{k}; if ischar(vark), switch(vark)
   case {'relax','relaxfactor'}
      relaxfactor = varargin{k+1};
   case {'maxit'},
      maxit = varargin{k+1};
   case {'c0','start','starting','init','initial'},
      c0 = varargin{k+1};
   case {'filter'},
      itfilter = varargin{k+1};
   case {'median'},
      itfilter = 'median';
   case {'taps','ntaps'},
      ntaps = varargin{k+1};
   case {'its2filter'},
      its2filter = varargin{k+1};
   case {'stoptol'} % stopping rule for relative increment
      tol = varargin{k+1};
   end, end, end
   if isnan(relaxfactor) & maxit>0,
      S = svd(full(Phi));
      relaxfactor = 1/max(S)^2;
   end
   if ischar(c0),
      initmethod = c0;
      c0 = NaN;
   elseif isnumeric(c0),
      if isnan(c0), initmethod = 'none';
      else, initmethod = 'givenvector';
      end
   end
case {'tichonov','tikhonov','tihonov','tychonov','tychonoff'},
   initmethod = 'Tichonov';
   itermethod = 'direct';
   if nvarargin > 0, alfa = varargin{1}; else, alfa = 0.2; end
case {'powerls','poweri'},
   initmethod = 'powerLS';
   itermethod = 'direct';
end

switch initmethod,
case {'derivative','deriv'},
   D = derivBsplinebasis(x,x,ptilde);
   Dbar = D(2:n-1,1:n);
   d = zeros(n,m);
   initmethod = 'constraintoptim';
case {'I','LS'},
   D = eye(n);
   Dbar = D(2:n-1,1:n);
   d = Dbar'*fbar;
   initmethod = 'constraintoptim';
end
switch initmethod,
case {'constraintoptim'},
   C = [Dbar'*Dbar Phibar';Phibar zeros(n-2,n-2)];
   c0 = C\[d;fbar];
   c0 = c0(1:n,1:m);
case {'Tichonov'},
   B = Phibar'*Phibar+alfa^2*eye(n);
   S = B\Phibar';
   c0 = S*fbar;
case {'powerLS'},
   Xtilde = allpowercoefsBsplines(x,ptilde);
   X = Phibar*Xtilde;
   S = sparsesolveAXisB(X',Xtilde',3,[1 (1:n-2) n-2])';
   c0 = S*fbar;
case {'pinv'},
   S = Phibar'/(Phibar*Phibar');
   c0 = S*fbar;
case {'none'},
   if sf(1) == n,
      c0 = f;
   elseif sf(1) == n-2,
      c0 = [f(1,1:m);f;f(n-2,1:m)];
   end
end

switch itermethod,
case {'direct'},
   c = c0;
case {'Landweber','landweber'},
   c = c0;
   it = 1;
   Phibar = sparse(Phibar);
   switch itfilter,
   case {'median'},
      if length(its2filter) == 1, 
         nfilter = ceil(maxit/its2filter);
         its2filter = its2filter*(1:nfilter);
      end
      if its2filter(1) > 1, 
         its2filter = [1 row(its2filter)];
              % after the first Landweber-step, we MUST filter, because c
              % might be an exact solver of Phibar*c-fbar = 0, so that cnew =
              % c, and iteration would stop immediately
      end
      its2filter = [row(its2filter) maxit+1];
      itfilt = 1;
   end
   landweberconvergence = false;
   while it <= maxit,
      cnew = c - relaxfactor*Phibar'*(Phibar*c-fbar);
      switch itfilter,
      case {'median'},
         if it<maxit & it>=its2filter(itfilt), % no filtering in the last step
            cnew = medianfilt(cnew,ntaps);
            itfilt = itfilt+1;
         end
      % case {'eye','I'}, do nothing
      end
      if m==1, if norm(c-cnew)/norm(c)<tol,
         it = maxit;
         landweberconvergence = true;
      end, end
      % note: if f has multiple columns, then the current implementation
      % provides no route to stop the iterations before the maximum number of
      % iterations has been reached.
      c = cnew;
      it = it+1;
   end
   if landweberconvergence == false,
      warning('Landweber iteration has not fully converged')
   end
end
